
///////////////////////////////////////////////////////////////////////////////
//
//	level.h
//

#ifndef _LEVEL_
#define _LEVEL_

///////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <deque>
#include "base.h"
#include "window.h"
#include "gdi.h"
#include "undo.h"
#include "scrbar.h"
#include "level_setting.h"
#include "level_tilemap.h"
#include "level_sprites.h"
#include "level_bananas.h"
#include "level_cameras.h"

class ProjectManager;
class LevelManager;


//	LevelSetting_UndoMethod: Xe[Wݒ̃AhDEhD
//---------------------------------------------------------
class LevelSetting_UndoMethod : public UndoMethod{
	LevelManager *level_manager;
	LevelSettingData data_before;
	LevelSettingData data_after;
	int level_num;
	virtual LPCSTR text_undo() const{ return "Xe[W̐ݒ"; }
	virtual LPCSTR text_redo() const{ return "Xe[W̐ݒ"; }
	virtual void undo();
	virtual void redo();
	void renew();
public:
	LevelSetting_UndoMethod(LevelManager *_level_manager,
		const LevelSettingData *_data_before, const LevelSettingData *_data_after, int _level_num);
	~LevelSetting_UndoMethod();
};


//	RecentLevelManager: ŋߊJXe[W̊Ǘ
//---------------------------------------------------------
class RecentLevelManager : public UI_Base, public RomData_Base{
	typedef std::deque<int> LevelNumbers;
	static const int LEVELS_MAX=16;
	Window *pwFrame;
	LevelManager *level_manager;
	LevelNumbers recent_levels;
public:
	RecentLevelManager(LevelManager *_level_manager, Window *_pwFrame);
	BOOL create();
	BOOL command(WPARAM wParam, LPARAM lParam);
	void level_opened(int level_num);
	void clear();
	void save_data();
};


enum LevelEditPage{
	LEVEL_UNKNOWN = -1,
	LEVEL_SPRITES = 0,
	LEVEL_BANANAS,
	LEVEL_CAMERAS
};


//	LevelEditInspector: Xe[WҏWCXyN^EBhE̊Ǘ
//---------------------------------------------------------
class LevelEditInspector : public MyWindow{

	static LPCSTR pAppName;
	static LPCSTR pClassName;
	virtual LRESULT Proc(HWND hWNd, UINT msg, WPARAM wParam, LPARAM lParam);

	HWND hwTab;
	TabPage **pages;
	int nPages;
	ProjectManager *pwFrame;
	void page_changed(int page);

public:
	static BOOL Init();
	LevelEditInspector(ProjectManager *_pwFrame);
	~LevelEditInspector();
	BOOL create(TabPage *(_pages[]), int _nPages);
	void save_config();
	void select_page(int page);
	void nc_activate(WPARAM wParam);

};



//	LevelManager: Xe[W֌WSʂ̊ǗEMapViewWindoẘǗ
//---------------------------------------------------------
class LevelManager : public MyWindow, public RomData_Base, public WindowWithBothScrollBars{
	friend class LevelSetting_UndoMethod;

private:

	int level_num;
	LevelSettingData setting;
	int setting_max_size;

	bool map_vert;
	int map_cx;
	int map_cy;
	WORD map_code;

	enum{
		DRAG_NONE,		// hbOłȂ
		DRAG_MAP,		// }bv(wi)hbO
		DRAG_SPRITE,	// XvCghbO
		DRAG_BANANA,	// oiihbO
		DRAG_CAMERA		// JhbO
	} mouse_drag;
	bool bDragging;
	POINT mouse_drag_point;

	BitmapBuffer buffer;
	MButtonPan mpan;
	RecentLevelManager recent_levels;

	ProjectManager *project;

public:
	
	LevelTilemapManager tilemap;
	LevelSpritesManager sprites;
	LevelBananasManager bananas;
	LevelCamerasManager cameras;
	LevelEditInspector inspector;

private:

	virtual LRESULT Proc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
	void load_level();
	void level_setting();
	void save_config();
	void redraw(const RECT *rtGlobal);
	virtual void scrolled_h(int nNewPos, UINT nNewPage, int nOldPos, UINT nOldPage);
	virtual void scrolled_v(int nNewPos, UINT nNewPage, int nOldPos, UINT nOldPage);
	void mouseover(LONG x, LONG y);
	void clicked(LONG x, LONG y);
	void unclicked(LONG x, LONG y);
	void dragging(LONG x, LONG y);

public:

	LevelManager(ProjectManager *_project);
	~LevelManager();

	BOOL create();
	void reset();
	void clear();
	void load_level(int _level_num);
	BOOL command(WPARAM wParam, LPARAM lParam);
	void redraw_default();
	void redraw_all();
	void set_state(bool bEnabled);
	void excluse_pages(LevelEditPage page_selected);
	void select_page(LevelEditPage page_to_select);
	WORD get_code() const{ return map_code; }
	void set_scroll_pos(int pdx, int pdy);
	bool is_point_in_screen(int pdx, int pdy);
	int get_level_num(){ return level_num; }


private:
	static LPCSTR pAppNameMapView;
	static LPCSTR pClassName;
public:
	static BOOL Init();

};





///////////////////////////////////////////////////////////////////////////////

#endif // _LEVEL_






