
///////////////////////////////////////////////////////////////////////////////
//
//	level_bananas.cpp
//

#include <windows.h>
#include <stdio.h>
#include "project.h"
#include "id.h"



void BananaAllocData::load_data(const ProjectManager *project, BANK bank, SADRS adrs, WORD code){
	DWORD data;
	project->read_data(&data, sizeof data, bank, adrs);
	type = (data & 0xFF000000) >> 24;
	fore = (data & 0x00800000) != 0;
	fore2 = (data & 0x00400000) != 0;
	switch (code){
		case 0x0000:
			pdx = ((data & 0x00007FF0) >> 4) * 8;		// 11bit
			pdy = ((data & 0x003F8000) >> 15) * 8;		// 7bit
			break;
		case 0001:
			pdx = ((data & 0x00003FF0) >> 4) * 8;		// 10bit
			pdy = ((data & 0x003FC000) >> 14) * 8;		// 8bit
			break;
		case 0x0004:
			pdx = ((data & 0x00000FF0) >> 4) * 8;		// 8bit
			pdy = ((data & 0x003FF000) >> 12) * 8;		// 10bit
			break;
		case 0x0006: case 0x0007:
			pdx = ((data & 0x00001FF0) >> 4) * 8;		// 9bit
			pdy = ((data & 0x003FE000) >> 13) * 8;		// 9bit
			break;
		case 0x0008:
			pdx = ((data & 0x0000FFF0) >> 4) * 8;		// 12bit
			pdy = ((data & 0x003F0000) >> 16) * 8;		// 6bit
			break;
		case 0x0009:
			pdx = ((data & 0x00001FF0) >> 4) * 8;		// 9bit
			pdy = ((data & 0x003FE000) >> 13) * 8;		// 9bit
			break;
		default:
			return;
	}
	return;
}

void BananaAllocData::normalize(WORD code){
	switch (code){
		case 0x0000:
			pdx &= 0x3FF8;		// 11bit
			pdy &= 0x03F8;		// 7bit
			break;
		case 0x0001:
			pdx &= 0x1FF8;		// 10bit
			pdy &= 0x07F8;		// 8bit
			break;
		case 0x0004:
			pdx &= 0x07F8;		// 8bit
			pdy &= 0x1FF8;		// 10bit
			break;
		case 0x0006: case 0x0007:
			pdx &= 0x0FF8;		// 9bit
			pdy &= 0x0FF8;		// 9bit
			break;
		case 0x0008:
			pdx &= 0x7FF8;		// 12bit
			pdy &= 0x01F8;		// 6bit
			break;
		case 0x0009:
			pdx &= 0x0FF8;		// 9bit
			pdy &= 0x0FF8;		// 9bit
			break;
	}
	return;
}

void BananaAllocData::store_data(ProjectManager *project, BANK bank, SADRS adrs, WORD code) const{
	DWORD data = type << 24;
	data |= (fore!=false) << 23;
	data |= (fore2!=false) << 22;
	switch (code){
		case 0x0000:
			data |= ((pdx/8) & 0x07FF) << 4;		// 11bit
			data |= ((pdy/8) & 0x00FF) << 15;		// 7bit
			break;
		case 0x0001:
			data |= ((pdx/8) & 0x03FF) << 4;		// 10bit
			data |= ((pdy/8) & 0x01FF) << 14;		// 8bit
			break;
		case 0x0004:
			data |= ((pdx/8) & 0x00FF) << 4;		// 8bit
			data |= ((pdy/8) & 0x07FF) << 12;		// 10bit
			break;
		case 0x0006: case 0x0007:
			data |= ((pdx/8) & 0x01FF) << 4;		// 9bit
			data |= ((pdy/8) & 0x07FF) << 13;		// 9bit
			break;
		case 0x0008:
			data |= ((pdx/8) & 0x0FFF) << 4;		// 12bit
			data |= ((pdy/8) & 0x00FF) << 16;		// 6bit
			break;
		case 0x0009:
			data |= ((pdx/8) & 0x01FF) << 4;		// 9bit
			data |= ((pdy/8) & 0x07FF) << 13;		// 9bit
			break;
		default:
			return;
	}
	project->write_data(&data, 4, bank, adrs);
	return;
}


//-----------------------------------------------------------------------------


void LevelBananasData::load_data(const ProjectManager *project, BANK bank, SADRS adrs, int level_num, WORD code){
	WORD adrs_this = project->read_word(bank, adrs + level_num*2);
	WORD adrs_next = project->read_word(bank, adrs + (level_num+1)*2);
	int size = (adrs_next - adrs_this) / BananaAllocData::size;
	SADRS p = adrs + adrs_this;
	BananaAllocData banana;

	bananas.clear();
	for (int i=0; i<size; i++){
		banana.load_data(project, bank, p, code);
		bananas.push_back(banana);
		p+=BananaAllocData::size;
	}
	return;
}

void LevelBananasData::store_data(ProjectManager *project, BANK bank, SADRS adrs, int level_num, WORD code) const{
	WORD adrs_this = project->read_word(bank, adrs + level_num*2);
	WORD adrs_next = project->read_word(bank, adrs + (level_num+1)*2);
	int size = (adrs_next - adrs_this) / BananaAllocData::size;
	SADRS p = adrs + adrs_this;

	std::list<BananaAllocData>::const_iterator i, i_end = bananas.end();
	int n;
	for (i=bananas.begin(), n=0; i!=i_end && n<size; i++, n++){
		(*i).store_data(project, bank, p, code);
		p+=BananaAllocData::size;
	}
	return;
}

void LevelBananasData::clear(){
	bananas.clear();
	return;
}







