
///////////////////////////////////////////////////////////////////////////////
//
//	level_cameras.cpp
//

#include "project.h"


void CameraAllocData::load_data(const ProjectManager *project, BANK bank, SADRS adrs, WORD code){
	BYTE data[size];
	project->read_data(data, sizeof data, bank, adrs);

	fr2	= (data[0] & 0x08) != 0;
	fr	= (data[0] & 0x04) != 0;
	fd2	= (data[0] & 0x02) != 0;
	fd	= (data[0] & 0x01) != 0;

	switch (code){
		case 0x0000:
			p1 = (data[4] & 0x80) != 0;
			p2 = (data[4] & 0x40) != 0;
			pdl = ((data[0]&0xF0) >> 4) | ((data[1]&0x7F) << 4);
			pdt = ((data[1]&0x80) >> 7) | ((data[2]&0x1F) << 1);
			pdr = ((data[2]&0xE0) >> 5) | ((data[3]&0xFF) << 3);
			pdb = data[4]&0x3F;
			break;
		case 0x0001:
			p1 = (data[4] & 0x80) != 0;
			p2 = (data[4] & 0x40) != 0;
			pdl = ((data[0]&0xF0) >> 4) | ((data[1]&0x3F) << 4);
			pdt = ((data[1]&0xC0) >> 6) | ((data[2]&0x1F) << 2);
			pdr = ((data[2]&0xE0) >> 5) | ((data[3]&0x7F) << 3);
			pdb = ((data[3]&0x80) >> 7) | ((data[4]&0x3F) << 1);
			break;
		case 0x0004:
			p1 = (data[4] & 0x80) != 0;
			p2 = (data[4] & 0x40) != 0;
			pdl = ((data[0]&0xF0) >> 4) | ((data[1]&0x07) << 4);
			pdt = ((data[1]&0xF8) >> 3) | ((data[2]&0x1F) << 5);
			pdr = ((data[2]&0xE0) >> 5) | ((data[3]&0x0F) << 3);
			pdb = ((data[3]&0xF0) >> 4) | ((data[4]&0x3F) << 4);
			break;
		case 0x0006: case 0x0007: case 0x0009:
			p1 = (data[2] & 0x80) != 0;
			p2 = (data[2] & 0x40) != 0;
			pdl = ((data[0]&0xF0) >> 4) | ((data[1]&0x1F) << 4);
			pdt = ((data[1]&0xE0) >> 5) | ((data[2]&0x3F) << 3);
			pdr = data[3] + pdl + 15;
			pdb = data[4] + pdt + 13;
			break;
		case 0x0008:
			p1 = (data[2] & 0x80) != 0;
			p2 = (data[2] & 0x40) != 0;
			pdl = ((data[0]&0xF0) >> 4) | ((data[1]&0xFF) << 4);
			pdt = data[2]&0x3F;
			pdr = ( data[3] | ((data[4]&0x03) << 8) ) + pdl + 15;
			pdb = (data[4]&0xFC) >> 2;
			break;
		default:
			return;
	}

	pdl = pdl * 8;
	pdt = pdt * 8;
	pdr = (pdr+1) * 8;
	pdb = (pdb+1) * 8;
	return;
}

void CameraAllocData::normalize(WORD code){
	pdl = pdl >> 3;
	pdt = pdt >> 3;
	pdr = (pdr >> 3) - 1;
	pdb = (pdb >> 3) - 1;
	switch (code){
		case 0x0000:
			pdl &= 0x07FF;
			pdt &= 0x003F;
			pdr &= 0x07FF;
			pdb &= 0x003F;
			break;
		case 0x0001:
			pdl &= 0x03FF;
			pdt &= 0x007F;
			pdr &= 0x03FF;
			pdb &= 0x007F;
			break;
		case 0x0004:
			pdl &= 0x007F;
			pdt &= 0x03FF;
			pdr &= 0x007F;
			pdb &= 0x03FF;
			break;
		case 0x0006: case 0x0007: case 0x0009:
			pdr = pdr - pdl - 15;
			pdb = pdb - pdt - 13;
			if (pdr < 0) pdr = 0;
			if (pdb < 0) pdb = 0;
			pdl &= 0x01FF;
			pdt &= 0x01FF;
			pdr &= 0x00FF;
			pdb &= 0x00FF;
			pdr = pdr + pdl + 15;
			pdb = pdb + pdt + 13;
			break;
		case 0x0008:
			pdr = pdr - pdl - 15;
			if (pdr < 0) pdr = 0;
			pdl &= 0x0FFF;
			pdt &= 0x003F;
			pdr &= 0x03FF;
			pdb &= 0x003F;
			pdr = pdr + pdl + 15;
			break;
	}
	pdl = pdl << 3;
	pdt = pdt << 3;
	pdr = (pdr+1) << 3;
	pdb = (pdb+1) << 3;
	return;
}

void CameraAllocData::store_data(ProjectManager *project, BANK bank, SADRS adrs, WORD code) const{
	BYTE data[size] = {0, 0, 0, 0, 0};
	data[0] = (fd!=0) | ((fd2!=0)<<1) | ((fr!=0)<<2) | ((fr2!=0)<<3);
	int odl = pdl >> 3;
	int odt = pdt >> 3;
	int odr = (pdr >> 3) - 1;
	int odb = (pdb >> 3) - 1;
	switch (code){
		case 0x0000:
			data[4] |= ((p1!=0)<<7) | ((p2!=0)<<6);
			data[0] |= (odl&0x000F) << 4;
			data[1] |= (odl&0x07F0) >> 4;
			data[1] |= (odt&0x0001) << 7;
			data[2] |= (odt&0x003E) >> 1;
			data[2] |= (odr&0x0007) << 5;
			data[3] |= (odr&0x07F8) >> 3;
			data[4] |= (odb&0x003F);
			break;
		case 0x0001:
			data[4] |= ((p1!=0)<<7) | ((p2!=0)<<6);
			data[0] |= (odl&0x000F) << 4;
			data[1] |= (odl&0x03F0) >> 4;
			data[1] |= (odt&0x0003) << 6;
			data[2] |= (odt&0x007C) >> 2;
			data[2] |= (odr&0x0007) << 5;
			data[3] |= (odr&0x03F8) >> 3;
			data[3] |= (odb&0x0001) << 7;
			data[4] |= (odb&0x007E) >> 1;
			break;
		case 0x0004:
			data[4] |= ((p1!=0)<<7) | ((p2!=0)<<6);
			data[0] |= (odl&0x000F) << 4;
			data[1] |= (odl&0x0070) >> 4;
			data[1] |= (odt&0x001F) << 3;
			data[2] |= (odt&0x03E0) >> 5;
			data[2] |= (odr&0x0007) << 5;
			data[3] |= (odr&0x0078) >> 3;
			data[3] |= (odb&0x000F) << 4;
			data[4] |= (odb&0x03F0) >> 4;
			break;
		case 0x0006: case 0x0007: case 0x0009:
			odr = odr - odl - 15;
			odb = odb - odt - 13;
			data[2] |= ((p1!=0)<<7) | ((p2!=0)<<6);
			data[0] |= (odl&0x000F) << 4;
			data[1] |= (odl&0x01F0) >> 4;
			data[1] |= (odt&0x0007) << 5;
			data[2] |= (odt&0x01F8) >> 3;
			data[3] |= (odr&0x00FF);
			data[4] |= (odb&0x00FF);
			break;
		case 0x0008:
			odr = odr - odl - 15;
			data[2] |= ((p1!=0)<<7) | ((p2!=0)<<6);
			data[0] |= (odl&0x000F) << 4;
			data[1] |= (odl&0x0FF0) >> 4;
			data[2] |= (odt&0x003F);
			data[3] |= (odr&0x00FF);
			data[4] |= (odr&0x0300) >> 8;
			data[4] |= (odb&0x003F) << 2;
			break;
		default:
			return;
	}
	project->write_data(data, sizeof data, bank, adrs);
	return;
}





//-----------------------------------------------------------------------------


void LevelCamerasData::load_data(const ProjectManager *project, BANK bank, SADRS adrs, int level_num, WORD code){
	WORD adrs_this = project->read_word(bank, adrs + level_num*2);
	WORD adrs_next = project->read_word(bank, adrs + (level_num+1)*2);
	int size = (adrs_next - adrs_this) / CameraAllocData::size;
	SADRS p = adrs + adrs_this;
	CameraAllocData camera;
	cameras.clear();
	for (int i=0; i<size; i++){
		camera.load_data(project, bank, p, code);
		cameras.push_back(camera);
		p+=CameraAllocData::size;
	}
	return;
}

void LevelCamerasData::store_data(ProjectManager *project, BANK bank, SADRS adrs, int level_num, WORD code) const{
	WORD adrs_this = project->read_word(bank, adrs + level_num*2);
	WORD adrs_next = project->read_word(bank, adrs + (level_num+1)*2);
	int size = (adrs_next - adrs_this) / CameraAllocData::size;
	SADRS p = adrs + adrs_this;
	std::list<CameraAllocData>::const_iterator i, i_end = cameras.end();
	int n;
	for (i=cameras.begin(), n=0; i!=i_end && n<size; i++, n++){
		(*i).store_data(project, bank, p, code);
		p+=CameraAllocData::size;
	}
	return;
}

void LevelCamerasData::clear(){
	cameras.clear();
	return;
}






