
///////////////////////////////////////////////////////////////////////////////
//
//	level_cameras.h
//

#ifndef _LEVEL_CAMERAS_
#define _LEVEL_CAMERAS_

///////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <list>
#include "window.h"
#include "base.h"


class LevelCamerasManager;
class LevelManager;
class ProjectManager;


//	JPݒ̃f[^(\Ă)
//---------------------------------------------------------
struct CameraAllocData{
	static const int size = 5;
	int pdl;	// [
	int pdt;	// [
	int pdr;	// E[
	int pdb;	// [
	bool fr;	// Eɑ
	bool fr2;	// Eɑ(2)
	bool fd;	// ɑ
	bool fd2;	// ɑ(2)
	bool p1;	// Param 1
	bool p2;	// Param 2
	void load_data(const ProjectManager *project, BANK bank, SADRS adrs, WORD code);
	void store_data(ProjectManager *project, BANK bank, SADRS adrs, WORD code) const;
	void normalize(WORD code);
};



//---------------------------------------------------------
class LevelCamerasData : public RomData_Base{
	std::list<CameraAllocData> cameras;
public:
	typedef std::list<CameraAllocData>::iterator iterator;
	typedef std::list<CameraAllocData>::reverse_iterator reverse_iterator;
	void load_data(const ProjectManager *project, BANK bank, SADRS adrs, int level_num, WORD code);
	void store_data(ProjectManager *project, BANK bank, SADRS adrs, int level_num, WORD code) const;
	void clear();
	iterator begin(){ return cameras.begin(); }
	iterator end(){ return cameras.end(); }
	reverse_iterator rbegin(){ return cameras.rbegin(); }
	reverse_iterator rend(){ return cameras.rend(); }
	size_t get_camera_count(){ return cameras.size(); }
};



//---------------------------------------------------------
class LevelCameraApply_UndoMethod : public UndoMethod, public RomData_Base{
	ProjectManager *project_manager;
	LevelManager *level_manager;
	LevelCamerasManager *cameras_manager;
	int level_num;
	int camera_num;
	CameraAllocData data_before;
	CameraAllocData data_after;
	virtual LPCSTR text_undo() const{ return "Change Camera"; }
	virtual LPCSTR text_redo() const{ return "Change Camera"; }
	virtual void undo();
	virtual void redo();
	void method_in_level(bool bUndo);
	void method_out_of_level(bool bUndo);
public:
	LevelCameraApply_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelCamerasManager *_cameras_manager, int _level_num, int _camera_num,
		const CameraAllocData *_data_before, const CameraAllocData *_data_after);
	~LevelCameraApply_UndoMethod();
};

class LevelCameraUpDown_UndoMethod : public UndoMethod, public RomData_Base{
	void method_in_level(bool bUp);
	void method_out_of_level(bool bUp);
protected:
	ProjectManager *project_manager;
	LevelManager *level_manager;
	LevelCamerasManager *cameras_manager;
	int level_num;
	int camera_former;
	int camera_latter;
	LevelCameraUpDown_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelCamerasManager *_cameras_manager, int _level_num, int _camera_former);
	~LevelCameraUpDown_UndoMethod();
	void method(bool bUp);
};

class LevelCameraUp_UndoMethod : public LevelCameraUpDown_UndoMethod{
	virtual LPCSTR text_undo() const{ return "Camera Up"; }
	virtual LPCSTR text_redo() const{ return "Camera Up"; }
	virtual void undo();
	virtual void redo();
public:
	LevelCameraUp_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelCamerasManager *_cameras_manager, int _level_num, int _camera_num);
	~LevelCameraUp_UndoMethod();
};

class LevelCameraDown_UndoMethod : public LevelCameraUpDown_UndoMethod{
	virtual LPCSTR text_undo() const{ return "Camera Down"; }
	virtual LPCSTR text_redo() const{ return "Camera Down"; }
	virtual void undo();
	virtual void redo();
public:
	LevelCameraDown_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelCamerasManager *_cameras_manager, int _level_num, int _camera_num);
	~LevelCameraDown_UndoMethod();
};

class LevelCameraSort_UndoMethod : public UndoMethod, public RomData_Base{
	ProjectManager *project_manager;
	LevelManager *level_manager;
	LevelCamerasManager *cameras_manager;
	int level_num;
	LevelCamerasData data_before;
	LevelCamerasData data_after;
	int camera_before;
	int camera_after;
	virtual LPCSTR text_undo() const{ return "Sort Cameras"; }
	virtual LPCSTR text_redo() const{ return "Sort Cameras"; }
	virtual void undo();
	virtual void redo();
	void method_in_level(bool bUndo);
	void method_out_of_level(bool bUndo);
public:
	LevelCameraSort_UndoMethod(ProjectManager *_project_manager, LevelManager *_level_manager,
		LevelCamerasManager *_cameras_manager, int _level_num, int _camera_before, int _camera_after,
		const LevelCamerasData &_data_before, const LevelCamerasData &_data_after);
	~LevelCameraSort_UndoMethod();
};



//---------------------------------------------------------
class LevelCamerasManager : public RomData_Base, public MyDialog, public TabPage{
	friend class LevelCameraApply_UndoMethod;
	friend class LevelCameraUpDown_UndoMethod;
	friend class LevelCameraSort_UndoMethod;

private:
	virtual LPCSTR get_text() const{ return "Cameras"; }
	virtual LPCSTR get_template() const{ return "CAMERA_EDIT"; }
	virtual BOOL Proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);

	static const int CAMERA_WIDTH = 28;
	static const int CAMERA_HEIGHT = 18;
	static const int CAMERA_LINEWIDTH = 5;
	HPEN	hpnCameraLine;
	HPEN	hpnCameraSel;
	HBRUSH	hbrCameraRect;
	HBRUSH	hbrCameraLine;
	HFONT	hftCameraText;
	bool bShowCameras;

	LevelCamerasData data;
	int isel;
	int level_num;
	enum {
		DRAG_NONE,
		DRAG_MOVE,
		DRAG_LEFT,
		DRAG_TOP,
		DRAG_RIGHT,
		DRAG_BOTTOM,
		DRAG_LEFT_TOP,
		DRAG_LEFT_BOTTOM,
		DRAG_RIGHT_TOP,
		DRAG_RIGHT_BOTTOM
	} bDragging;
	int dragging_x;
	int dragging_y;
	int drag_temp_x;
	int drag_temp_y;
	int drag_const_width;
	int drag_const_height;

	LevelManager *level;
	ProjectManager *project;

	void set_menuitem_state();
	void sel_changed(int sel);
	void jump(int sel);
	void focus_on_camera(int sel);
	void cam_apply(int sel);
	void cam_up(int sel);
	void cam_down(int sel);
	void cam_sort(int sel);
	void get_camera_text(char *dst, size_t size, const CameraAllocData *camera);
	LevelCamerasData::iterator get_iterator(int iIndex);
	void input_camera_data(CameraAllocData *camera);
	void output_camera_data(const CameraAllocData *camera, int iIndex);

public:

	LevelCamerasManager(LevelManager *_level, ProjectManager *_project);
	~LevelCamerasManager();
	virtual BOOL create(HWND hwTab);
	BOOL create_sys();
	void save_data();
	void load(int _level_num);
	void clear();
	void draw(BitmapBuffer *buf, const RECT *rtRedraw);
	void draw_f(BitmapBuffer *buf, const RECT *rtRedraw);
	BOOL command(WPARAM wParam, LPARAM lParam);
	void set_state(bool bEnabled);
	bool mouseover(int pdx, int pdy);
	bool accepts_click(int pdx, int pdy);
	void dragging(int pdx, int pdy);
	void release_click(bool bApply);
	void release_sel();

	LADRS get_camera_levels_adrs() const{
		DWORD dw_adrs = GetPrivateProfileInt("Cameras", "Levels", 0x3DC20C, RomInfoFileName);
		LADRS ladrs;
		ladrs.bank = (BANK)((dw_adrs&0xff0000)>>16);
		ladrs.adrs = (SADRS)(dw_adrs&0xffff);
		return ladrs;
	}

};






///////////////////////////////////////////////////////////////////////////////

#endif // _LEVEL_CAMERAS_







