
///////////////////////////////////////////////////////////////////////////////
//
//	level_sprites.cpp
//

#include <stdio.h>
#include "id.h"
#include "project.h"


void SpriteAllocData::load_data(const ProjectManager *project, BANK bank, SADRS adrs){
	param = project->read_word(bank, adrs);
	pdx   = (int)project->read_word(bank, adrs+2) - 256;
	pdy   = (int)project->read_word(bank, adrs+4) - 256;
	type  = project->read_word(bank, adrs+6);
	return;
}

void SpriteAllocData::store_data(ProjectManager *project, BANK bank, SADRS adrs) const{
	project->write_word(param, bank, adrs);
	project->write_word((WORD)(pdx + 256), bank, adrs+2);
	project->write_word((WORD)(pdy + 256), bank, adrs+4);
	project->write_word(type, bank, adrs+6);
	return;
}



int LevelSpritesData::get_size() const{
	return sprites.size() * SpriteAllocData::size + 2;
}

int LevelSpritesData::load_data(const ProjectManager *project, BANK bank, SADRS adrs){
	SpriteAllocData sprite;
	WORD wData;
	int i = 0;
	sprites.clear();
	while (1){
		wData = project->read_word(bank, adrs+i);
		if (wData == 0x0000){
			i += 2;
			break;
		}
		sprite.load_data(project, bank, adrs+i);
		sprites.push_back(sprite);
		i += SpriteAllocData::size;
	}
	return i;
}

int LevelSpritesData::store_data(ProjectManager *project, BANK bank, SADRS adrs, int size_max) const{
	int p = 0;
	std::list<SpriteAllocData>::const_iterator i, i_end = sprites.end();
	for (i=sprites.begin(); i!=i_end; i++){
		if (p+8 > size_max){
			return -1;
		}
		i->store_data(project, bank, adrs+p);
		p += SpriteAllocData::size;
	}
	if (p+2 > size_max){
		return -1;
	}
	project->write_word(0x0000, bank, adrs+p);
	p += 2;
	return p;
}

void LevelSpritesData::clear(){
	sprites.clear();
	return;
}

