
///////////////////////////////////////////////////////////////////////////////
//
//	level_sprui.cpp
//

#include <stdio.h>
#include "id.h"
#include "project.h"
#include "sprite_select.h"

#ifdef _MSC_VER
#include <uxtheme.h>
#endif

//-----------------------------------------------------------------------------

void LevelSpritesManager::get_sprite_text(char *dst, size_t size, const SpriteAllocData *sprite){
	char s[128];
	get_sprite_name(s, sizeof s, sprite->type);
	sprintf_s(dst, size, "%04X-%04X: %s", sprite->param, sprite->type, s);
	return;
}

void LevelSpritesManager::get_sprite_name(char *dst, size_t size, WORD type){
	char s[16];
	sprintf_s(s, sizeof s, "%04X", type);
	GetPrivateProfileString("SpriteName", s, "?", dst, size, SpriteInfoFileName);
	return;
}

LevelSpritesManager::LevelSpritesManager(LevelManager *_level, ProjectManager *_project)
: level(_level), project(_project), bDragging(false)
{
	hpnSpriteLine = NULL;
	hpnSpriteSel  = NULL;
	hbrSpriteRect = NULL;
	hbrSpriteLine = NULL;
	hftSpriteText = NULL;
}

LevelSpritesManager::~LevelSpritesManager(){
	if (hpnSpriteLine) DeleteObject(hpnSpriteLine);
	if (hpnSpriteSel)  DeleteObject(hpnSpriteSel);
	if (hbrSpriteRect) DeleteObject(hbrSpriteRect);
	if (hbrSpriteLine) DeleteObject(hbrSpriteLine);
	if (hftSpriteText) DeleteObject(hftSpriteText);
}

BOOL LevelSpritesManager::create_sys(){
	hpnSpriteLine = CreatePen(PS_SOLID, 1, RGB(255, 0, 0));
	hpnSpriteSel  = CreatePen(PS_SOLID, 3, RGB(255, 128, 128));
	hbrSpriteRect = CreateSolidBrush(RGB(0, 0, 0));
	hbrSpriteLine = CreateSolidBrush(RGB(255, 0, 0));
	hftSpriteText = CreateFont(12, 0, 0, 0, FW_REGULAR, FALSE, FALSE, FALSE, SHIFTJIS_CHARSET,
		OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, PROOF_QUALITY, FIXED_PITCH | FF_MODERN, "lr SVbN");
	bShowSprites = GetPrivateProfileInt("MapView", "ShowSprites", TRUE, ConfigFileName) != 0;
	set_menuitem_state();
	return TRUE;
}

BOOL LevelSpritesManager::create(HWND hwTab){
	if (!create_dialog(hwTab)) return FALSE;
	return TRUE;
}

void LevelSpritesManager::set_menuitem_state(){
	MENUITEMINFO mi;
	mi.cbSize	= sizeof mi;
	mi.fMask	= MIIM_STATE;
	mi.fState	= bShowSprites ? MFS_CHECKED : MFS_UNCHECKED;
	SetMenuItemInfo(GetMenu(project->get_hWnd()), IDM_VIEW_SPRITES, FALSE, &mi);
	return;
}

void LevelSpritesManager::save_data(){
	char s[16];
	sprintf_s(s, sizeof s, "%d", bShowSprites ? 1 : 0);
	WritePrivateProfileString("MapView", "ShowSprites", s, ConfigFileName);
	return;
}

void LevelSpritesManager::clear(){
	LPCSTR empty = "";
	data.clear();
	SendMessage(GetDlgItem(hWnd, IDC_LIST0), LB_RESETCONTENT, 0L, 0L);
	SetDlgItemText(hWnd, IDC_EDIT0, empty);
	SetDlgItemText(hWnd, IDC_EDIT1, empty);
	SetDlgItemText(hWnd, IDC_EDIT2, empty);
	SetDlgItemText(hWnd, IDC_EDIT3, empty);
	SetDlgItemText(hWnd, IDC_EDIT4, empty);
	SetDlgItemText(hWnd, IDC_EDIT5, empty);
	return;
}

void LevelSpritesManager::load(int _level_num){
	WORD left, right, sdr;
	char s[128], s2[128];
	level_num = _level_num;
	HWND hwList = GetDlgItem(hWnd, IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	clear();
	// XvCgzu̓ǂݍ
	int level_sprites_count = GetPrivateProfileInt("Sprites", "LevelCount", 224, RomInfoFileName);
	SADRS adrs_sprites = project->read_sadrs(0x3E, 2*level_num);
	if (data.load_data(project, 0x3E, adrs_sprites) == -1){
		MessageBox(project->get_hWnd(), "Failed to load Sprite Data", "Error",
			MB_OK | MB_ICONEXCLAMATION);
	}else{
		right = adrs_sprites + data.get_size();
		left = GetPrivateProfileInt("Sprites", "AvailableEnd", 0x9500, RomInfoFileName);
		for (int i=0; i<level_sprites_count; i++){
			sdr = project->read_sadrs(0x3E, 2*i);
			if (sdr >= right && sdr < left) left = sdr;
		}
		max_size = left - adrs_sprites;
	}
	LevelSpritesData::iterator i, i_end = data.end();
	int n;
	for (i=data.begin(), n=0; i!=i_end; i++, n++){
		get_sprite_text(s, sizeof s, &(*i));
		sprintf_s(s2, sizeof s2, "#%d: %s", n, s);
		SendMessage(hwList, LB_INSERTSTRING, (WPARAM)n, (LPARAM)s2);
	}
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	isel = -1;
	return;
}

void LevelSpritesManager::set_state(bool bEnabled){
	EnableWindow(GetDlgItem(hWnd, IDC_LIST0), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT0), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT1), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT2), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT3), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT4), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_EDIT5), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_BUTTON0), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_BUTTON1), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_BUTTON2), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_BUTTON3), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_BUTTON4), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_BUTTON5), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_BUTTON6), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC0), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC1), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC2), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC3), bEnabled);
	EnableWindow(GetDlgItem(hWnd, IDC_STATIC4), bEnabled);
	return;
}

BOOL LevelSpritesManager::Proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam){
	switch (msg){
		case WM_INITDIALOG:
			{
#ifdef _MSC_VER
				HMODULE hMod;
				hMod = LoadLibrary("uxtheme.dll");
				if (hMod)
				{
					typedef HRESULT (WINAPI *EnableThemeDialogTexturePROC)(HWND, DWORD);
					EnableThemeDialogTexturePROC pEnableThemeDialogTexture;
					pEnableThemeDialogTexture = (EnableThemeDialogTexturePROC)GetProcAddress(hMod, "EnableThemeDialogTexture");
					if (pEnableThemeDialogTexture)
						pEnableThemeDialogTexture(hWnd, ETDT_ENABLETAB);
					FreeLibrary(hMod);
				}
#endif
				SetWindowLong(GetDlgItem(hDlg, IDC_EDIT0), GWL_EXSTYLE, 0);
				//SetWindowLong(GetDlgItem(hDlg, IDC_EDIT5), GWL_EXSTYLE, 0);
			}
			return TRUE;
		case WM_COMMAND:
			switch (LOWORD(wParam)){
				case IDC_LIST0:
					if (HIWORD(wParam) == LBN_SELCHANGE){
						int sel = SendMessage(GetDlgItem(hDlg, IDC_LIST0), LB_GETCURSEL, 0, 0);
						sel_changed(sel);
						if (bShowSprites){
							isel = sel;
							level->excluse_pages(LEVEL_SPRITES);
							level->redraw_default();
						}
						return TRUE;
					}
					if (HIWORD(wParam) == LBN_DBLCLK){
						jump(SendDlgItemMessage(hDlg, IDC_LIST0, LB_GETCURSEL, 0, 0));
						return TRUE;
					}
					break;
				case IDC_EDIT4:
					if (HIWORD(wParam) == EN_CHANGE){
						type_changed();
						return TRUE;
					}
					break;
				case IDC_BUTTON0:	// ...
					select_sprite();
					return TRUE;
				case IDC_BUTTON1:	// Kp
					spr_apply(SendDlgItemMessage(hDlg, IDC_LIST0, LB_GETCURSEL, 0, 0));
					return TRUE;
				case IDC_BUTTON2:	// ǉ
					spr_add(SendDlgItemMessage(hDlg, IDC_LIST0, LB_GETCURSEL, 0, 0));
					return TRUE;
				case IDC_BUTTON3:	// 폜
					spr_delete(SendDlgItemMessage(hDlg, IDC_LIST0, LB_GETCURSEL, 0, 0));
					return TRUE;
				case IDC_BUTTON4:	// 
					spr_up(SendDlgItemMessage(hDlg, IDC_LIST0, LB_GETCURSEL, 0, 0));
					return TRUE;
				case IDC_BUTTON5:	// 
					spr_down(SendDlgItemMessage(hDlg, IDC_LIST0, LB_GETCURSEL, 0, 0));
					return TRUE;
				case IDC_BUTTON6:	// ɂ
					spr_sort(SendDlgItemMessage(hDlg, IDC_LIST0, LB_GETCURSEL, 0, 0));
					return TRUE;
			}
			break;
			/*
		case WM_CTLCOLORDLG:
			return (BOOL)GetStockObject(NULL_BRUSH);
		case WM_CTLCOLORBTN:
		case WM_CTLCOLOREDIT:
		//case WM_CTLCOLORLISTBOX:
		//case WM_CTLCOLORMSGBOX:
		//case WM_CTLCOLORSCROLLBAR:
		case WM_CTLCOLORSTATIC:
			//break;
			return (BOOL)GetStockObject(NULL_BRUSH);//*/
	}
	return FALSE;
}

void LevelSpritesManager::select_sprite(){
	SpriteSelectDialog sprite_sel;
	char s[256];
	int val;
	GetDlgItemText(hWnd, IDC_EDIT4, s, sizeof s);
	if (sscanf_s(s, "%x", &val) < 1) val = 0;
	WORD type = (WORD)val;
	val = sprite_sel.dialog_box(project, type);
	if (val == -1) return;
	type = (WORD)val;
	sprintf_s(s, sizeof s, "%04X", type);
	SetDlgItemText(hWnd, IDC_EDIT4, s);
	return;
}

void LevelSpritesManager::type_changed(){
	char s[256];
	int val;
	GetDlgItemText(hWnd, IDC_EDIT4, s, sizeof s);
	if (sscanf_s(s, "%x", &val) < 1) val = 0;
	get_sprite_name(s, sizeof s, (WORD)val);
	SetDlgItemText(hWnd, IDC_EDIT5, s);
	return;
}

void LevelSpritesManager::spr_apply(int sel){
	char s[256], s2[256];
	LevelSpritesData::iterator i = get_iterator(sel);
	if (i == data.end()) return;
	SpriteAllocData data_before = *i;
	input_sprite_data(&(*i));
	// ROMf[^ɔf
	SADRS adrs_sprite = project->read_sadrs(0x3E, 2*level_num) + sel*SpriteAllocData::size;
	(*i).store_data(project, 0x3E, adrs_sprite);
	// Xgɔf
	get_sprite_text(s2, sizeof s2, &(*i));
	sprintf_s(s, sizeof s, "#%d: %s", sel, s2);
	HWND hwList = GetDlgItem(hWnd, IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)sel, 0);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)sel, (LPARAM)s);
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)sel, 0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	level->redraw_default();
	// Undoobt@֒ǉ
	LevelSpriteApply_UndoMethod *undo = new LevelSpriteApply_UndoMethod(project, level, this,
		level_num, sel, &data_before, &*i);
	project->undo_buffer.add_method(undo);
	return;
}

void LevelSpritesManager::spr_add(int sel){
	char s[256], s2[128];
	int n;
	if (data.get_size() + SpriteAllocData::size > max_size){
		sprintf_s(s, sizeof s, "Unable to add any more object to this level."
			"", data.get_size(), max_size);
		MessageBox(project->get_hWnd(), s, "Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}
	LevelSpritesData::iterator i_sel = get_iterator(sel), i, i_end = data.end();
	HWND hwList = GetDlgItem(hWnd, IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for (i=i_sel; i!=i_end; i++){
		SendMessage(hwList, LB_DELETESTRING, (WPARAM)sel, 0);
	}
	// ݓ͂Ălf[^쐬
	SpriteAllocData sprite;
	input_sprite_data(&sprite);
	i_sel = data.insert(i_sel, sprite);
	if (sel == -1){
		sel = data.get_sprite_count()-1;
		get_sprite_text(s2, sizeof s2, &sprite);
		sprintf_s(s, sizeof s, "#%d: %s", sel, s2);
		SendMessage(hwList, LB_INSERTSTRING, (WPARAM)sel, (LPARAM)s);
	}else{
		for (i=i_sel, n=sel; i!=i_end; i++, n++){
			get_sprite_text(s2, sizeof s2, &(*i));
			sprintf_s(s, sizeof s, "#%d: %s", n, s2);
			SendMessage(hwList, LB_INSERTSTRING, (WPARAM)n, (LPARAM)s);
		}
	}
	if (bShowSprites){
		isel = sel;
		level->excluse_pages(LEVEL_SPRITES);
	}
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)sel, 0);
	output_sprite_data(&*i_sel, sel);
	// ROMɔf
	SADRS adrs_sprites = project->read_sadrs(0x3E, 2*level_num);
	data.store_data(project, 0x3E, adrs_sprites, max_size);
	level->redraw_default();
	// Undoobt@֒ǉ
	LevelSpriteAdd_UndoMethod *undo = new LevelSpriteAdd_UndoMethod(project, level, this,
		level_num, sel, &sprite);
	project->undo_buffer.add_method(undo);
	return;
}

void LevelSpritesManager::spr_delete(int sel){
	char s[128], s2[128];
	int n;
	LevelSpritesData::iterator i_sel = get_iterator(sel), i, i_end = data.end();
	if (i_sel == i_end) return;
	SpriteAllocData sprite = *i_sel;
	// Undoobt@֒ǉ
	LevelSpriteDelete_UndoMethod *undo = new LevelSpriteDelete_UndoMethod(project, level, this,
		level_num, sel, &sprite);
	project->undo_buffer.add_method(undo);
	HWND hwList = GetDlgItem(hWnd, IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for (i=i_sel; i!=i_end; i++){
		SendMessage(hwList, LB_DELETESTRING, (WPARAM)sel, 0);
	}
	if ((i_sel = data.erase(i_sel)) == i_end){
		sel--;
		if (sel != -1){
			output_sprite_data(&(data.back()), sel);
		}
		if (isel != -1) isel = sel;
	}else{
		for (i=i_sel, n=sel; i!=i_end; i++, n++){
			get_sprite_text(s, sizeof s, &(*i));
			sprintf_s(s2, sizeof s2, "#%d: %s", n, s);
			SendMessage(hwList, LB_INSERTSTRING, (WPARAM)n, (LPARAM)s2);
		}
		output_sprite_data(&*i_sel, sel);
	}
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	if (sel != -1){
		SendMessage(hwList, LB_SETCURSEL, (WPARAM)sel, 0);
	}
	// ROMɔf
	SADRS adrs_sprites = project->read_sadrs(0x3E, 2*level_num);
	data.store_data(project, 0x3E, adrs_sprites, max_size);
	level->redraw_default();
	return;
}

void LevelSpritesManager::spr_up(int sel){
	char s[128], s2[128];
	if (sel <= 0) return;
	LevelSpritesData::iterator i_sel = get_iterator(sel), i, i_end = data.end();
	if (i_sel == i_end) return;
	// Undoobt@֒ǉ
	LevelSpriteUp_UndoMethod *undo = new LevelSpriteUp_UndoMethod(project, level, this, level_num, sel);
	project->undo_buffer.add_method(undo);
	SpriteAllocData sprite_temp = *i_sel;
	i = i_sel--;
	*i = *i_sel;
	*i_sel = sprite_temp;
	sel--;
	if (isel != -1) isel--;
	HWND hwList = GetDlgItem(hWnd, IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)sel, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)sel, 0);
	get_sprite_text(s, sizeof s, &(*i_sel));
	sprintf_s(s2, sizeof s2, "#%d: %s", sel, s);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)sel, (LPARAM)s2);
	get_sprite_text(s, sizeof s, &(*i));
	sprintf_s(s2, sizeof s2, "#%d: %s", sel+1, s);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)(sel+1), (LPARAM)s2);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)sel, 0);
	// ROMɔf
	SADRS adrs_sprite = project->read_sadrs(0x3E, 2*level_num) + sel*SpriteAllocData::size;
	(*i_sel).store_data(project, 0x3E, adrs_sprite);
	(*i).store_data(project, 0x3E, adrs_sprite + SpriteAllocData::size);
	sprintf_s(s, sizeof s, "%d", sel);
	SetDlgItemText(hWnd, IDC_EDIT0, s);
	level->redraw_default();
	return;
}

void LevelSpritesManager::spr_down(int sel){
	char s[128], s2[128];
	if (sel < 0 || sel >= (int)(data.get_sprite_count())-1) return;
	LevelSpritesData::iterator i_sel = get_iterator(sel), i, i_end = data.end();
	if (i_sel == i_end) return;
	// Undoobt@֒ǉ
	LevelSpriteDown_UndoMethod *undo = new LevelSpriteDown_UndoMethod(project, level, this, level_num, sel);
	project->undo_buffer.add_method(undo);
	SpriteAllocData sprite_temp = *i_sel;
	i = i_sel++;
	*i = *i_sel;
	*i_sel = sprite_temp;
	sel++;
	if (isel != -1) isel++;
	HWND hwList = GetDlgItem(hWnd, IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)sel, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)(sel-1), 0);
	get_sprite_text(s, sizeof s, &(*i));
	sprintf_s(s2, sizeof s2, "#%d: %s", sel-1, s);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)(sel-1), (LPARAM)s2);
	get_sprite_text(s, sizeof s, &(*i_sel));
	sprintf_s(s2, sizeof s2, "#%d: %s", sel, s);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)sel, (LPARAM)s2);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)sel, 0);
	// ROMɔf
	SADRS adrs_sprite = project->read_sadrs(0x3E, 2*level_num) + sel*SpriteAllocData::size;
	(*i_sel).store_data(project, 0x3E, adrs_sprite);
	(*i).store_data(project, 0x3E, adrs_sprite - SpriteAllocData::size);
	sprintf_s(s, sizeof s, "%d", sel);
	SetDlgItemText(hWnd, IDC_EDIT0, s);
	level->redraw_default();
	return;
}

void LevelSpritesManager::spr_sort(int sel){
	int n, m;
	char s[256], s2[256];
	HWND hwList = GetDlgItem(hWnd, IDC_LIST0);
	LevelSpritesData data_before(data);
	int sel_before = sel;
	SpriteAllocData sprite_temp;
	LevelSpritesData::iterator i1, i2, i_end = data.end();
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for (i1=data.begin(), n=0; i1!=i_end; i1++, n++){
		for (i2=i1++, m=n+1; i1!=i_end; i1++, m++){
			if (((*i2).pdx == (*i1).pdx) ? ((*i2).pdy > (*i1).pdy) : ((*i2).pdx > (*i1).pdx)){
				sprite_temp = *i2;
				*i2 = *i1;
				*i1 = sprite_temp;
				if (n == sel){
					sel = m;
				}else if (m == sel){
					sel = n;
				}
			}
		}
		i1 = i2;
		SendMessage(hwList, LB_DELETESTRING, (WPARAM)n, 0);
		get_sprite_text(s, sizeof s, &(*i1));
		sprintf_s(s2, sizeof s2, "#%d: %s", n, s);
		SendMessage(hwList, LB_INSERTSTRING, (WPARAM)n, (LPARAM)s2);
	}
	if (isel != -1) isel = sel;
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)sel, 0);
	// ROMɔf
	SADRS adrs_sprites = project->read_sadrs(0x3E, 2*level_num);
	data.store_data(project, 0x3E, adrs_sprites, max_size);
	level->redraw_default();
	if (sel != -1){
		sprintf_s(s, sizeof s, "%d", sel);
		SetDlgItemText(hWnd, IDC_EDIT0, s);
	}
	// Undoobt@֒ǉ
	LevelSpriteSort_UndoMethod *undo = new LevelSpriteSort_UndoMethod(project, level, this, level_num,
		sel_before, sel, data_before, data);
	project->undo_buffer.add_method(undo);
	return;
}

void LevelSpritesManager::jump(int sel){
	LevelSpritesData::iterator i = get_iterator(sel);
	if (i == data.end()) return;
	level->set_scroll_pos((*i).pdx, (*i).pdy);
	return;
}

void LevelSpritesManager::focus_on_sprite(int sel){
	if (bShowSprites){
		isel = sel;
		LevelSpritesData::iterator i = get_iterator(sel);
		if (i == data.end()) return;
		if (!level->is_point_in_screen((*i).pdx, (*i).pdy))
			level->set_scroll_pos((*i).pdx, (*i).pdy);
	}
	level->redraw_default();
	return;
}

void LevelSpritesManager::input_sprite_data(SpriteAllocData *sprite){
	char s[256];
	int int_val;
	GetDlgItemText(hWnd, IDC_EDIT1, s, sizeof s);	// p[^
	if (sscanf_s(s, "%x", &int_val) < 1) int_val = 0;
	(*sprite).param = int_val;
	GetDlgItemText(hWnd, IDC_EDIT2, s, sizeof s);	// X
	if (sscanf_s(s, "%d", &int_val) < 1) int_val = 0;
	(*sprite).pdx = int_val;
	GetDlgItemText(hWnd, IDC_EDIT3, s, sizeof s);	// Y
	if (sscanf_s(s, "%d", &int_val) < 1) int_val = 0;
	(*sprite).pdy = int_val;
	GetDlgItemText(hWnd, IDC_EDIT4, s, sizeof s);	// 
	if (sscanf_s(s, "%x", &int_val) < 1) int_val = 0;
	(*sprite).type = int_val;
	return;
}

void LevelSpritesManager::output_sprite_data(const SpriteAllocData *sprite, int iIndex){
	char s[64], s2[256];
	sprintf_s(s, sizeof s, "%d", iIndex);
	SetDlgItemText(hWnd, IDC_EDIT0, s);
	sprintf_s(s, sizeof s, "%04X", (*sprite).param);
	SetDlgItemText(hWnd, IDC_EDIT1, s);
	sprintf_s(s, sizeof s, "%d", (*sprite).pdx);
	SetDlgItemText(hWnd, IDC_EDIT2, s);
	sprintf_s(s, sizeof s, "%d", (*sprite).pdy);
	SetDlgItemText(hWnd, IDC_EDIT3, s);
	sprintf_s(s, sizeof s, "%04X", (*sprite).type);
	SetDlgItemText(hWnd, IDC_EDIT4, s);
	get_sprite_name(s2, sizeof s2, (*sprite).type);
	SetDlgItemText(hWnd, IDC_EDIT5, s2);
	return;
}

void LevelSpritesManager::sel_changed(int sel){
	if (sel == -1) return;
	LevelSpritesData::iterator i = get_iterator(sel);
	if (i == data.end()) return;
	output_sprite_data(&(*i), sel);
	return;
}

LevelSpritesData::iterator LevelSpritesManager::get_iterator(int iIndex){
	LevelSpritesData::iterator i, i_end = data.end();
	if (iIndex == -1) return i_end;
	int n;
	for (i=data.begin(), n=0; i!=i_end; i++, n++){
		if (n == iIndex) return i;
	}
	return i_end;
}

bool LevelSpritesManager::mouseover(int pdx, int pdy){
	if (!bShowSprites) return false;
	int x, y, n;
	LevelSpritesData::reverse_iterator i, i_end = data.rend();
	for (i=data.rbegin(), n=data.get_sprite_count()-1; i!=i_end; i++, n--){
		x = (*i).pdx;
		y = (*i).pdy;
		if (pdx >= x - SPRITE_WIDTH/2 && pdx < x + SPRITE_WIDTH/2 && pdy >= y-SPRITE_HEIGHT && pdy < y){
			SetCursor(LoadCursor(NULL, IDC_SIZEALL));
			return true;
		}
	}
	return false;
}

bool LevelSpritesManager::accepts_click(int pdx, int pdy){
	if (!bShowSprites) return false;
	int x, y, n;
	LevelSpritesData::reverse_iterator i, i_end = data.rend();
	for (i=data.rbegin(), n=data.get_sprite_count()-1; i!=i_end; i++, n--){
		x = (*i).pdx;
		y = (*i).pdy;
		if (pdx >= x - SPRITE_WIDTH/2 && pdx < x + SPRITE_WIDTH/2 && pdy >= y-SPRITE_HEIGHT && pdy < y){
			isel = n;
			SendMessage(GetDlgItem(hWnd, IDC_LIST0), LB_SETCURSEL, (WPARAM)isel, 0);
			sel_changed(n);
			bDragging = true;
			dragging_x = pdx - x;
			dragging_y = pdy - y;
			drag_temp_x = x;
			drag_temp_y = y;
			return true;
		}
	}
	isel = -1;
	return false;
}

void LevelSpritesManager::release_click(bool bApply){
	LevelSpritesData::iterator i = get_iterator(isel);
	if (i == data.end()) return;
	if (bApply) spr_apply(isel);
	bDragging = false;
	return;
}

void LevelSpritesManager::dragging(int pdx, int pdy){
	char s[256];
	LevelSpritesData::iterator i = get_iterator(isel);
	if (i == data.end()) return;
	drag_temp_x = pdx - dragging_x;
	drag_temp_y = pdy - dragging_y;
	sprintf_s(s, sizeof s, "%d", drag_temp_x);
	SetDlgItemText(hWnd, IDC_EDIT2, s);
	sprintf_s(s, sizeof s, "%d", drag_temp_y);
	SetDlgItemText(hWnd, IDC_EDIT3, s);
	return;
}

void LevelSpritesManager::release_sel(){
	isel = -1;
	return;
}




BOOL LevelSpritesManager::command(WPARAM wParam, LPARAM lParam){
	if (LOWORD(wParam) == IDM_VIEW_SPRITES){
		bShowSprites = !bShowSprites;
		set_menuitem_state();
		if (!bShowSprites){
			isel = -1;
		}
		level->redraw_default();
		return TRUE;
	}
	return FALSE;
}

void LevelSpritesManager::draw(BitmapBuffer *buf, const RECT *rtRedraw){
	if (!bShowSprites) return;
	char s[64];
	int x, y, n;
	POINT points[3];
	LevelSpritesData::iterator i, i_end = data.end();
	buf->select_object(hftSpriteText);
	SetBkMode(buf->get_HDC(), TRANSPARENT);
	SetTextColor(buf->get_HDC(), RGB(255, 0, 0));
	for (i=data.begin(), n=0; i!=i_end; i++, n++){
		if (bDragging && n==isel) continue;
		x = (*i).pdx - rtRedraw->left;
		y = (*i).pdy - rtRedraw->top;
		buf->select_object(hpnSpriteLine);
		buf->select_object(hbrSpriteRect);
		buf->rectangle(x - SPRITE_WIDTH/2, y - SPRITE_HEIGHT, x + SPRITE_WIDTH/2, y);
		points[0].x = x - 6;
		points[0].y = y;
		points[1].x = x;
		points[1].y = y - 6;
		points[2].x = x + 6;
		points[2].y = y;
		buf->select_object(GetStockObject(NULL_PEN));
		buf->select_object(hbrSpriteLine);
		Polygon(buf->get_HDC(), points, 3);
		sprintf_s(s, sizeof s, "#%d", n);
		TextOut(buf->get_HDC(), x-SPRITE_WIDTH/2+3, y-SPRITE_HEIGHT+3, s, strlen(s));
		sprintf_s(s, sizeof s, "%04X", (*i).type);
		TextOut(buf->get_HDC(), x-SPRITE_WIDTH/2+3, y-SPRITE_HEIGHT+16, s, strlen(s));
		if (n == isel){
			buf->select_object(hpnSpriteSel);
			buf->select_object(GetStockObject(NULL_BRUSH));
			buf->rectangle(x - SPRITE_WIDTH/2-2, y - SPRITE_HEIGHT-2, x + SPRITE_WIDTH/2+2, y+2);
		}
	}
	return;
}

void LevelSpritesManager::draw_f(BitmapBuffer *buf, const RECT *rtRedraw){
	char s[64];
	int x, y;
	POINT points[3];
	if (!bShowSprites) return;
	if (!bDragging) return;
	LevelSpritesData::iterator i = get_iterator(isel);
	if (i == data.end()) return;
	x = drag_temp_x - rtRedraw->left;
	y = drag_temp_y - rtRedraw->top;
	buf->select_object(hpnSpriteLine);
	buf->select_object(hbrSpriteRect);
	buf->rectangle(x - SPRITE_WIDTH/2, y - SPRITE_HEIGHT, x + SPRITE_WIDTH/2, y);
	points[0].x = x - 6;
	points[0].y = y;
	points[1].x = x;
	points[1].y = y - 6;
	points[2].x = x + 6;
	points[2].y = y;
	buf->select_object(GetStockObject(NULL_PEN));
	buf->select_object(hbrSpriteLine);
	Polygon(buf->get_HDC(), points, 3);
	sprintf_s(s, sizeof s, "#%d", isel);
	TextOut(buf->get_HDC(), x-SPRITE_WIDTH/2+3, y-SPRITE_HEIGHT+3, s, strlen(s));
	sprintf_s(s, sizeof s, "%04X", (*i).type);
	TextOut(buf->get_HDC(), x-SPRITE_WIDTH/2+3, y-SPRITE_HEIGHT+16, s, strlen(s));
	buf->select_object(hpnSpriteSel);
	buf->select_object(GetStockObject(NULL_BRUSH));
	buf->rectangle(x - SPRITE_WIDTH/2-2, y - SPRITE_HEIGHT-2, x + SPRITE_WIDTH/2+2, y+2);
	return;
}











