
///////////////////////////////////////////////////////////////////////////////
//
//	level_sprum.cpp
//

#include <windows.h>
#include <stdio.h>
#include "project.h"
#include "id.h"



LevelSpriteApply_UndoMethod::LevelSpriteApply_UndoMethod(
	ProjectManager *_project_manager,
	LevelManager *_level_manager,
	LevelSpritesManager *_sprites_manager,
	int _level_num,
	int _sprite_num,
	const SpriteAllocData *_data_before,
	const SpriteAllocData *_data_after
	)
	: project_manager(_project_manager), level_manager(_level_manager), sprites_manager(_sprites_manager),
	level_num(_level_num), sprite_num(_sprite_num), data_before(*_data_before), data_after(*_data_after)
{
}

LevelSpriteApply_UndoMethod::~LevelSpriteApply_UndoMethod(){}

void LevelSpriteApply_UndoMethod::undo(){
	(this->*(level_manager->get_level_num() == level_num ? &LevelSpriteApply_UndoMethod::method_in_level
		: &LevelSpriteApply_UndoMethod::method_out_of_level))(true);
	return;
}

void LevelSpriteApply_UndoMethod::redo(){
	(this->*(level_manager->get_level_num() == level_num ? &LevelSpriteApply_UndoMethod::method_in_level
		: &LevelSpriteApply_UndoMethod::method_out_of_level))(false);
	return;
}

void LevelSpriteApply_UndoMethod::method_in_level(bool bUndo){
	char s[128], s2[128];
	level_manager->select_page(LEVEL_SPRITES);
	level_manager->excluse_pages(LEVEL_SPRITES);
	LevelSpritesData::iterator i = sprites_manager->get_iterator(sprite_num);
	*i = (bUndo ? data_before : data_after);
	SADRS adrs_sprite = project_manager->read_sadrs(0x3E, 2*level_num) + sprite_num*SpriteAllocData::size;
	(*i).store_data(project_manager, 0x3E, adrs_sprite);
	HWND hwList = GetDlgItem(sprites_manager->hWnd, IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)sprite_num, 0);
	sprites_manager->get_sprite_text(s, sizeof s, &*i);
	sprintf_s(s2, sizeof s2, "#%d: %s", sprite_num, s);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)sprite_num, (LPARAM)s2);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)sprite_num, 0);
	sprites_manager->output_sprite_data(&*i, sprite_num);
	sprites_manager->focus_on_sprite(sprite_num);
	return;
}

void LevelSpriteApply_UndoMethod::method_out_of_level(bool bUndo){
	SADRS adrs_sprite = project_manager->read_sadrs(0x3E, 2*level_num) + sprite_num*SpriteAllocData::size;
	(bUndo ? data_before : data_after).store_data(project_manager, 0x3E, adrs_sprite);
	level_manager->load_level(level_num);
	level_manager->select_page(LEVEL_SPRITES);
	SendDlgItemMessage(sprites_manager->hWnd, IDC_LIST0, LB_SETCURSEL, (WPARAM)sprite_num, 0);
	sprites_manager->sel_changed(sprite_num);
	sprites_manager->focus_on_sprite(sprite_num);
	return;
}

//=================================================================================================


LevelSpriteAddDelete_UndoMethod::LevelSpriteAddDelete_UndoMethod(
	ProjectManager *_project_manager,
	LevelManager *_level_manager,
	LevelSpritesManager *_sprites_manager,
	int _level_num,
	int _sprite_num,
	const SpriteAllocData *_data
	)
	: project_manager(_project_manager), level_manager(_level_manager), sprites_manager(_sprites_manager),
	level_num(_level_num), sprite_num(_sprite_num), data(*_data)
{
}

LevelSpriteAddDelete_UndoMethod::~LevelSpriteAddDelete_UndoMethod(){}

void LevelSpriteAddDelete_UndoMethod::do_add(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(true);
	}else{
		method_out_of_level(true);
	}
	return;
}

void LevelSpriteAddDelete_UndoMethod::do_delete(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(false);
	}else{
		method_out_of_level(false);
	}
	return;
}

void LevelSpriteAddDelete_UndoMethod::method_in_level(bool bAdd){
	int n;
	char s[256], s2[128];
	level_manager->select_page(LEVEL_SPRITES);
	level_manager->excluse_pages(LEVEL_SPRITES);
	LevelSpritesData::iterator i_sel = sprites_manager->get_iterator(sprite_num);
	LevelSpritesData::iterator i_end = sprites_manager->data.end(), i;
	HWND hwList = GetDlgItem(sprites_manager->get_hWnd(), IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for (i=i_sel; i!=i_end; i++){
		SendMessage(hwList, LB_DELETESTRING, (WPARAM)sprite_num, 0);
	}
	i_sel = (bAdd ? sprites_manager->data.insert(i_sel, data) : sprites_manager->data.erase(i_sel));
	for (i=i_sel, n=sprite_num; i!=i_end; i++, n++){
		sprites_manager->get_sprite_text(s2, sizeof s2, &(*i));
		sprintf_s(s, sizeof s, "#%d: %s", n, s2);
		SendMessage(hwList, LB_INSERTSTRING, (WPARAM)n, (LPARAM)s);
	}
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)sprite_num, 0);
	sprites_manager->output_sprite_data(&*i_sel, sprite_num);
	// ROMɔf
	SADRS adrs_sprites = project_manager->read_sadrs(0x3E, 2*level_num);
	sprites_manager->data.store_data(project_manager, 0x3E, adrs_sprites, sprites_manager->max_size);
	sprites_manager->focus_on_sprite(sprite_num);
	return;
}

void LevelSpriteAddDelete_UndoMethod::method_out_of_level(bool bAdd){
	SADRS adrs_sel = project_manager->read_sadrs(0x3E, 2*level_num) + sprite_num*SpriteAllocData::size;
	SADRS adrs_next = adrs_sel + SpriteAllocData::size;
	int size = 0;
	while (project_manager->read_word(0x3E, adrs_sel + (size++)*SpriteAllocData::size));
	if (bAdd){
		memmove_s(project_manager->get_pointer(0x3E, adrs_next), size*SpriteAllocData::size + 2,
			project_manager->get_pointer(0x3E, adrs_sel), size*SpriteAllocData::size + 2);
		data.store_data(project_manager, 0x3E, adrs_sel);
	}else{
		memmove_s(project_manager->get_pointer(0x3E, adrs_sel), size*SpriteAllocData::size + 2,
			project_manager->get_pointer(0x3E, adrs_next), size*SpriteAllocData::size + 2);
	}
	level_manager->load_level(level_num);
	level_manager->select_page(LEVEL_SPRITES);
	SendDlgItemMessage(sprites_manager->hWnd, IDC_LIST0, LB_SETCURSEL, (WPARAM)sprite_num, 0);
	sprites_manager->sel_changed(sprite_num);
	sprites_manager->focus_on_sprite(sprite_num);
	return;
}

//-------------------------------------------------------------------------------------------------


LevelSpriteAdd_UndoMethod::LevelSpriteAdd_UndoMethod(
	ProjectManager *_project_manager,
	LevelManager *_level_manager,
	LevelSpritesManager *_sprites_manager,
	int _level_num,
	int _sprite_num,
	const SpriteAllocData *_data
	)
	: LevelSpriteAddDelete_UndoMethod(_project_manager, _level_manager, _sprites_manager,
	_level_num, _sprite_num, _data)
{
}

LevelSpriteAdd_UndoMethod::~LevelSpriteAdd_UndoMethod(){}

void LevelSpriteAdd_UndoMethod::undo(){
	do_delete();
}

void LevelSpriteAdd_UndoMethod::redo(){
	do_add();
}


//-------------------------------------------------------------------------------------------------

LevelSpriteDelete_UndoMethod::LevelSpriteDelete_UndoMethod(
	ProjectManager *_project_manager,
	LevelManager *_level_manager,
	LevelSpritesManager *_sprites_manager,
	int _level_num,
	int _sprite_num,
	const SpriteAllocData *_data
	)
	: LevelSpriteAddDelete_UndoMethod(_project_manager, _level_manager, _sprites_manager,
	_level_num, _sprite_num, _data)
{
}

LevelSpriteDelete_UndoMethod::~LevelSpriteDelete_UndoMethod(){}

void LevelSpriteDelete_UndoMethod::undo(){
	do_add();
}

void LevelSpriteDelete_UndoMethod::redo(){
	do_delete();
}

//=================================================================================================

LevelSpriteUpDown_UndoMethod::LevelSpriteUpDown_UndoMethod(
	ProjectManager *_project_manager,
	LevelManager *_level_manager,
	LevelSpritesManager *_sprites_manager,
	int _level_num,
	int _sprite_former
	)
	: project_manager(_project_manager), level_manager(_level_manager), sprites_manager(_sprites_manager),
	level_num(_level_num), sprite_former(_sprite_former), sprite_latter(_sprite_former+1)
{
}

LevelSpriteUpDown_UndoMethod::~LevelSpriteUpDown_UndoMethod(){}

void LevelSpriteUpDown_UndoMethod::do_up(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(true);
	}else{
		method_out_of_level(true);
	}
	return;
}

void LevelSpriteUpDown_UndoMethod::do_down(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(false);
	}else{
		method_out_of_level(false);
	}
	return;
}

void LevelSpriteUpDown_UndoMethod::method_in_level(bool bUp){
	char s[256], s2[128];
	level_manager->select_page(LEVEL_SPRITES);
	level_manager->excluse_pages(LEVEL_SPRITES);
	LevelSpritesData::iterator i_former = sprites_manager->get_iterator(sprite_former);
	LevelSpritesData::iterator i_latter = sprites_manager->get_iterator(sprite_latter);
	HWND hwList = GetDlgItem(sprites_manager->get_hWnd(), IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)sprite_latter, 0);
	SendMessage(hwList, LB_DELETESTRING, (WPARAM)sprite_former, 0);
	SpriteAllocData data_temp = *i_former;
	*i_former = *i_latter;
	*i_latter = data_temp;
	sprites_manager->get_sprite_text(s2, sizeof s2, &*i_former);
	sprintf_s(s, sizeof s, "#%d: %s", sprite_former, s2);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)sprite_former, (LPARAM)s);
	sprites_manager->get_sprite_text(s2, sizeof s2, &*i_latter);
	sprintf_s(s, sizeof s, "#%d: %s", sprite_latter, s2);
	SendMessage(hwList, LB_INSERTSTRING, (WPARAM)sprite_latter, (LPARAM)s);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	int sprite_num = bUp ? sprite_former : sprite_latter;
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)sprite_num, 0);
	sprites_manager->output_sprite_data(&*(bUp ? i_former : i_latter), sprite_num);
	// ROMɔf
	SADRS adrs_sprites = project_manager->read_sadrs(0x3E, 2*level_num);
	SADRS adrs_former = adrs_sprites + sprite_former*SpriteAllocData::size;
	SADRS adrs_latter = adrs_sprites + sprite_latter*SpriteAllocData::size;
	(*i_former).store_data(project_manager, 0x3E, adrs_former);
	(*i_latter).store_data(project_manager, 0x3E, adrs_latter);
	sprites_manager->focus_on_sprite(sprite_num);
	return;
}

void LevelSpriteUpDown_UndoMethod::method_out_of_level(bool bUp){
	SADRS adrs_sprites = project_manager->read_sadrs(0x3E, 2*level_num);
	SADRS adrs_former = adrs_sprites + sprite_former*SpriteAllocData::size;
	SADRS adrs_latter = adrs_sprites + sprite_latter*SpriteAllocData::size;
	SpriteAllocData data1, data2;
	data1.load_data(project_manager, 0x3E, adrs_former);
	data2.load_data(project_manager, 0x3E, adrs_latter);
	data1.store_data(project_manager, 0x3E, adrs_latter);
	data2.store_data(project_manager, 0x3E, adrs_former);
	level_manager->load_level(level_num);
	level_manager->select_page(LEVEL_SPRITES);
	int sprite_num = bUp ? sprite_former : sprite_latter;
	SendDlgItemMessage(sprites_manager->hWnd, IDC_LIST0, LB_SETCURSEL, (WPARAM)sprite_num, 0);
	sprites_manager->sel_changed(sprite_num);
	sprites_manager->focus_on_sprite(sprite_num);
	return;
}

//-------------------------------------------------------------------------------------------------

LevelSpriteUp_UndoMethod::LevelSpriteUp_UndoMethod(
	ProjectManager *_project_manager,
	LevelManager *_level_manager,
	LevelSpritesManager *_sprites_manager,
	int _level_num,
	int _sprite_num
	)
	: LevelSpriteUpDown_UndoMethod(_project_manager, _level_manager, _sprites_manager,
	_level_num, _sprite_num-1)
{
}

LevelSpriteUp_UndoMethod::~LevelSpriteUp_UndoMethod(){}


void LevelSpriteUp_UndoMethod::undo(){
	do_down();
}

void LevelSpriteUp_UndoMethod::redo(){
	do_up();
}

//-------------------------------------------------------------------------------------------------

LevelSpriteDown_UndoMethod::LevelSpriteDown_UndoMethod(
	ProjectManager *_project_manager,
	LevelManager *_level_manager,
	LevelSpritesManager *_sprites_manager,
	int _level_num,
	int _sprite_num
	)
	: LevelSpriteUpDown_UndoMethod(_project_manager, _level_manager, _sprites_manager,
	_level_num, _sprite_num)
{
}

LevelSpriteDown_UndoMethod::~LevelSpriteDown_UndoMethod(){}


void LevelSpriteDown_UndoMethod::undo(){
	do_up();
}

void LevelSpriteDown_UndoMethod::redo(){
	do_down();
}


//=================================================================================================


LevelSpriteSort_UndoMethod::LevelSpriteSort_UndoMethod(
	ProjectManager *_project_manager,
	LevelManager *_level_manager,
	LevelSpritesManager *_sprites_manager,
	int _level_num,
	int _sprite_before,
	int _sprite_after,
	const LevelSpritesData &_data_before,
	const LevelSpritesData &_data_after
	)
	: project_manager(_project_manager), level_manager(_level_manager), sprites_manager(_sprites_manager),
	level_num(_level_num), sprite_before(_sprite_before), sprite_after(_sprite_after),
	data_before(_data_before), data_after(_data_after)
{
}

LevelSpriteSort_UndoMethod::~LevelSpriteSort_UndoMethod(){}

void LevelSpriteSort_UndoMethod::undo(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(true);
	}else{
		method_out_of_level(true);
	}
}

void LevelSpriteSort_UndoMethod::redo(){
	if (level_manager->get_level_num() == level_num){
		method_in_level(false);
	}else{
		method_out_of_level(false);
	}
}

void LevelSpriteSort_UndoMethod::method_in_level(bool bUndo){
	int n;
	char s[256], s2[128];
	level_manager->select_page(LEVEL_SPRITES);
	level_manager->excluse_pages(LEVEL_SPRITES);
	LevelSpritesData::iterator i, i_end;
	HWND hwList = GetDlgItem(sprites_manager->get_hWnd(), IDC_LIST0);
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for (i=sprites_manager->data.begin(), i_end=sprites_manager->data.end(); i!=i_end; i++){
		SendMessage(hwList, LB_DELETESTRING, (WPARAM)0, 0);
	}
	sprites_manager->data = bUndo ? data_before : data_after;
	for (i=sprites_manager->data.begin(), i_end=sprites_manager->data.end(), n=0; i!=i_end; i++, n++){
		sprites_manager->get_sprite_text(s2, sizeof s2, &(*i));
		sprintf_s(s, sizeof s, "#%d: %s", n, s2);
		SendMessage(hwList, LB_INSERTSTRING, (WPARAM)n, (LPARAM)s);
	}
	SendMessage(hwList, WM_SETREDRAW, (WPARAM)TRUE, 0);
	int sprite_num = bUndo ? sprite_before : sprite_after;
	SendMessage(hwList, LB_SETCURSEL, (WPARAM)sprite_num, 0);
	sprites_manager->sel_changed(sprite_num);
	// ROMɔf
	SADRS adrs_sprites = project_manager->read_sadrs(0x3E, 2*level_num);
	sprites_manager->data.store_data(project_manager, 0x3E, adrs_sprites, sprites_manager->max_size);
	sprites_manager->focus_on_sprite(sprite_num);
	return;
}

void LevelSpriteSort_UndoMethod::method_out_of_level(bool bUndo){
	SADRS adrs_sprites = project_manager->read_sadrs(0x3E, 2*level_num);
	(bUndo? data_before: data_after).store_data(project_manager, 0x3E, adrs_sprites, sprites_manager->max_size);
	level_manager->load_level(level_num);
	level_manager->select_page(LEVEL_SPRITES);
	int sprite_num = bUndo ? sprite_before : sprite_after;
	SendDlgItemMessage(sprites_manager->hWnd, IDC_LIST0, LB_SETCURSEL, (WPARAM)sprite_num, 0);
	sprites_manager->sel_changed(sprite_num);
	sprites_manager->focus_on_sprite(sprite_num);
	return;
}






