
///////////////////////////////////////////////////////////////////////////////
//
//	level_tilemap.cpp
//

#include "id.h"
#include "project.h"


LevelTilemapManager::LevelTilemapManager(LevelManager *_level, ProjectManager *_project)
: level(_level), project(_project)
{
	tilemap = NULL;
	tilemap_org = NULL;
	actionmap = NULL;
}

LevelTilemapManager::~LevelTilemapManager(){
	if (tilemap) delete[] tilemap;
	if (tilemap_org) delete[] tilemap_org;
	if (actionmap) delete[] actionmap;
	DeleteObject(hpnNormal);
	DeleteObject(hpnType1);
	DeleteObject(hpnType2);
	DeleteObject(hpnType3);
	DeleteObject(hbrNormal);
	DeleteObject(hbrType1);
	DeleteObject(hbrType2);
	DeleteObject(hbrType3);
}

BOOL LevelTilemapManager::create(){
	// yEuV̍쐬
	hpnNormal = CreatePen(PS_SOLID, 1, RGB(128, 128, 128));
	hpnType1  = CreatePen(PS_SOLID, 1, RGB(128, 0, 0));
	hpnType2  = CreatePen(PS_SOLID, 1, RGB(128, 64, 0));
	hpnType3  = CreatePen(PS_SOLID, 1, RGB(64, 64, 64));
	hbrNormal = CreateSolidBrush(RGB(128, 128, 128));
	hbrType1  = CreateSolidBrush(RGB(128, 0, 0));
	hbrType2  = CreateSolidBrush(RGB(128, 64, 0));
	hbrType3  = CreateSolidBrush(RGB(64, 64, 64));
	// ANV`bv̓ǂݍ
	//char str[MAX_PATH];
	//sprintf_s(str, sizeof str, "%schips\\action.bmp", ExeDirectory);
	//action_chip.create_and_load(GetDC(level->get_hWnd()), str);
	action_chip.create(GetDC(level->get_hWnd()), 16*32*4, 64*32);
	iShowTilemap = (ShowTilemapType)GetPrivateProfileInt("MapView", "ShowTilemap", TRUE, ConfigFileName);
	set_menuitem_state();
	return TRUE;
}

void LevelTilemapManager::save_data(){
	char s[16];
	sprintf_s(s, sizeof s, "%d", iShowTilemap);
	WritePrivateProfileString("MapView", "ShowTilemap", s, ConfigFileName);
	return;
}

void LevelTilemapManager::set_menuitem_state(){
	MENUITEMINFO mi;
	mi.cbSize	= sizeof mi;
	mi.fMask	= MIIM_STATE;
	mi.fState	= iShowTilemap==MAP_HIDE ? MFS_CHECKED : MFS_UNCHECKED;
	SetMenuItemInfo(GetMenu(project->get_hWnd()), IDM_VIEW_MAP_HIDE, FALSE, &mi);
	mi.fState	= iShowTilemap==MAP_TILEMAP ? MFS_CHECKED : MFS_UNCHECKED;
	SetMenuItemInfo(GetMenu(project->get_hWnd()), IDM_VIEW_MAP_TILEMAP, FALSE, &mi);
	mi.fState	= iShowTilemap==MAP_ACTION ? MFS_CHECKED : MFS_UNCHECKED;
	SetMenuItemInfo(GetMenu(project->get_hWnd()), IDM_VIEW_MAP_ACTION, FALSE, &mi);
	return;
}


void LevelTilemapManager::load(
	const LevelSettingData *setting,
	HDC hdc_wnd
	)
{

	int map_num;
	BANK bMap;
	LADRS lTilemap, lActionmap;
	//SADRS sMapLoadFunc, sMapAlloc, sMapTypePointers, sActionmapSizes;
	//SADRS sTilemapPointers, sMapAllocPointers, sActionmapPointers;
	SADRS sMapABInfoPointers;
	SADRS sMapAInfo, sMapBInfo;
	WORD wMapC1, wMapC2, wMapC3, map_left, map_top, map_right, map_bottom;
	if (setting->type == 0x0004 || setting->type == 0x0005){
		map_valid = false;
		map_cx = 0;
		map_cy = 0;
		map_code = 0x0000;
		map_vert = false;
		return;
	}
	// }bṽAhX̎擾

	bMap				= (BANK)GetPrivateProfileInt("Maps", "MapInfoBank", 0x37, RomInfoFileName);
	sMapABInfoPointers	= (SADRS)GetPrivateProfileInt("Maps", "MapABInfo", 0xB01B, RomInfoFileName);

	map_num				= project->read_byte(0x3D, setting->style + 8);

	sMapAInfo			= (SADRS)project->read_word(bMap, sMapABInfoPointers + map_num*4);
	sMapBInfo			= (SADRS)project->read_word(bMap, sMapABInfoPointers + map_num*4 + 2);

	/*
	sMapLoadFunc = (SADRS)GetPrivateProfileInt("Maps", "MapLoadFunc", 0xBCA8, RomInfoFileName);
	map_num				= project->read_byte(0x3D, setting->style + 0x12);
	sMapTypePointers	= project->read_sadrs(0x35, sMapLoadFunc + 0x56);
	sMapAllocPointers	= project->read_sadrs(0x35, sMapLoadFunc + 0x50);
	sTilemapPointers	= project->read_sadrs(0x35, sMapLoadFunc + 0x0C);
	sActionmapPointers	= project->read_sadrs(0x35, sMapLoadFunc + 0x1E);
	sActionmapSizes		= project->read_sadrs(0x35, sMapLoadFunc + 0x3A);
	wMapType			= project->read_word(0x35, sMapTypePointers + map_num*2);
	sMapAlloc			= project->read_sadrs(0x35, sMapAllocPointers + map_num*2);
	lTilemap			= project->read_ladrs_normal(0x35, sTilemapPointers + map_num*3);
	lActionmap			= project->read_ladrs_normal(0x35, sActionmapPointers + map_num*3);
	//*/
	// ANV}bv̓ǂݍ
	/*
	if (lActionmap.get() != 0x000000
		&& (actionmap_size = project->read_word(0x35, sActionmapSizes + map_num*2))!=0
		)
	{
		if (actionmap) delete[] actionmap;
		actionmap = new DWORD[actionmap_size];
		project->read_data(actionmap, actionmap_size*4, lActionmap);
		// ANV`bv̍\
		write_action_chips();
	}
	//*/
	lActionmap.adrs = lActionmap.bank = 0;
	actionmap = NULL;
	lTilemap.adrs = project->read_word(bMap, sMapAInfo);
	lTilemap.bank = (BANK)project->read_byte(bMap, sMapAInfo + 2);

	if (lTilemap.adrs != 0x0000 && lTilemap.bank != 0x00){
		lTilemap.adrs += 0x0009;
		lTilemap.bank = (lTilemap.bank|0x80) - 0xC0;
	}
	
	// }bv̍W̎擾
	wMapC1		= project->read_word(bMap, sMapAInfo + 3);
	wMapC2		= project->read_word(bMap, sMapAInfo + 5);
	wMapC3		= project->read_word(bMap, sMapAInfo + 7);
	map_left	= project->read_word(bMap, sMapAInfo + 9 + setting->map_index*10) / 32;
	map_right	= project->read_word(bMap, sMapAInfo + 11 + setting->map_index*10) / 32;
	map_top		= project->read_word(bMap, sMapAInfo + 13 + setting->map_index*10) / 32;
	map_bottom	= project->read_word(bMap, sMapAInfo + 15 + setting->map_index*10) / 32;
	map_code	= project->read_word(bMap, sMapAInfo + 17 + setting->map_index*10);
	map_cx = map_right - map_left;
	map_cy = map_bottom - map_top;

	//wMapType = 0x0000;

	WORD wMapCX = wMapC2;
	WORD wMapCY = wMapC3;
	int height;

	/*
	wMapCX		= project->read_word(0x35, sMapAlloc);
	wMapCY		= project->read_word(0x35, sMapAlloc + 2);
	map_left	= project->read_word(0x35, sMapAlloc + 4 + setting->map_index*10) / 32;
	map_right	= project->read_word(0x35, sMapAlloc + 6 + setting->map_index*10) / 32;
	map_top		= project->read_word(0x35, sMapAlloc + 8 + setting->map_index*10) / 32;
	map_bottom	= project->read_word(0x35, sMapAlloc + 10 + setting->map_index*10) / 32;
	map_code	= project->read_word(0x35, sMapAlloc + 12 + setting->map_index*10);
	map_cx = map_right - map_left;
	map_cy = map_bottom - map_top;
	//*/
	// n`f[^{̂̓ǂݍ
	if (lTilemap.adrs != 0x0000 && lTilemap.bank != 0x00/*wMapType & 0x0020*/){
		// n`
		map_valid = true;
		WORD *buf_map = new WORD[65536];
		int size;
		if (1/*wMapType & 0x0400*/){
			// k
			extern int ExpandData(unsigned char *buf, unsigned int buf_size, const unsigned char *buf_src);
			size = ExpandData((unsigned char *)buf_map, 65536,
				(const unsigned char *)project->get_pointer(lTilemap));
			if (size == -1){
				delete[] buf_map;
				MessageBox(project->get_hWnd(), "}bvf[^̓WJɎs܂B", "G[", MB_OK | MB_ICONEXCLAMATION);
				map_vert = (wMapCX & 0x0FFF) != 0;
				goto NO_TILEMAP;
			}
		}else{
			// kȂ
			project->read_data(buf_map, 65536, lTilemap);
		}
		// }bv̐؂蔲
		WORD tile, h_flip, v_flip;
		if (tilemap) delete[] tilemap;
		tilemap = new WORD[65536];
		if (tilemap_org) delete[] tilemap_org;
		tilemap_org = new WORD[65536];
		if (map_code==0x0004 || map_code==0x0005 || map_code==0x0006 || map_code==0x0007 || map_code==0x0009){
			// c
			height = wMapCX / 32;
			for (int y=0, gy=map_top; gy<map_bottom; y++, gy++){
				for (int x=0, gx=map_left; gx<map_right; x++, gx++){
					tile = buf_map[gx+gy*height];
					h_flip = tile & 0x4000;
					v_flip = tile & 0x8000;
					tilemap_org[x+y*map_cx] = tile;
					tilemap[x+y*map_cx] = (tile&0x000F) | ((h_flip!=0)<<4)
						| ((v_flip!=0)<<5) | ((tile&0x03F0)<<4);
				}
			}
			map_vert = true;
		}else{
			// 
			height = wMapCY / 32;
			for (int x=0, gx=map_left; gx<map_right; x++, gx++){
				for (int y=0, gy=map_top; gy<map_bottom; y++, gy++){
					tile = buf_map[gy+gx*height];
					h_flip = tile & 0x4000;
					v_flip = tile & 0x8000;
					tilemap_org[x+y*map_cx] = tile;
					tilemap[x+y*map_cx] = (tile&0x000F) | ((h_flip!=0)<<4)
						| ((v_flip!=0)<<5) | ((tile&0x03F0)<<4);
				}
			}
			map_vert = false;
		}
		delete[] buf_map;
		// `bv摜̓ǂݍ
		char str[MAX_PATH], str2[64];
		sprintf_s(str, sizeof str, "%02X", map_num);
		GetPrivateProfileString("MapChip", str, "", str2, sizeof str2, RomInfoFileName);
		sprintf_s(str, sizeof str, "%schips\\%s", ExeDirectory, str2);
		map_chip.create_and_load(hdc_wnd, str);
	}else{
		map_cx = 8;
		map_cy = 8;
		map_vert = true;
NO_TILEMAP:
		// n`Ȃ
		map_valid = false;
	}
	//*/
	return;
}



void LevelTilemapManager::draw(BitmapBuffer *buf, const RECT *rtRedraw){
	//WORD action;
	WORD tile;
	//bool vflip, hflip;
	//bool aflip;
	if (iShowTilemap == MAP_ACTION && (!tilemap_org || !actionmap)) return;
	if (map_valid && iShowTilemap!=MAP_HIDE){
		int left = rtRedraw->left/32;
		int top = rtRedraw->top/32;
		int right = min(map_cx-1, rtRedraw->right/32+1);
		int bottom = min(map_cy-1, rtRedraw->bottom/32+1);
		int left_b = rtRedraw->left%32;
		int top_b = rtRedraw->top%32;
		for (int x=left, sx=0; x<=right; x++, sx++){
			for (int y=top, sy=0; y<=bottom; y++, sy++){
				if (iShowTilemap == MAP_TILEMAP){
					BitBlt(buf->get_HDC(), sx*32-left_b, sy*32-top_b, 32, 32,
						map_chip.get_HDC(), (tilemap[x+y*map_cx]&0x00FF)*32,
						(tilemap[x+y*map_cx]&0xFF00)/8, SRCCOPY);
				}else if (iShowTilemap == MAP_ACTION){
					tile = tilemap[x + y*map_cx];
					BitBlt(buf->get_HDC(), sx*32-left_b, sy*32-top_b, 32, 32,
						action_chip.get_HDC(), (tile&0x00FF)*32,
						(tile&0xFF00)/8, SRCCOPY);
					/*
					tile = tilemap_org[x + y*map_cx];
					hflip = (tile&0x4000) != 0;
					vflip = (tile&0x8000) != 0;
					tile &= 0x03FF;
					if (tile >= actionmap_size) tile = 0;
					action = (hflip ? HIWORD(actionmap[tile]) : LOWORD(actionmap[tile]));
					aflip = ((action&0x8000)!=0) ^ (hflip!=0);
					action &= 0x7F;
					StretchBlt(buf->get_HDC(), sx*32-left_b + (aflip?15:0), sy*32-top_b + (vflip?31:0),
						(aflip ? -16 : 16), (vflip ? -32 : 32),
						action_chip.get_HDC(), (action%16)*16, (action/16)*32, 16, 32, SRCCOPY);
					action = (hflip ? LOWORD(actionmap[tile]) : HIWORD(actionmap[tile]));
					aflip = ((action&0x8000)!=0) ^ (hflip!=0);
					action &= 0x7F;
					StretchBlt(buf->get_HDC(), sx*32-left_b + (aflip?31:16), sy*32-top_b + (vflip?31:0),
						(aflip ? -16 : 16), (vflip ? -32 : 32),
						action_chip.get_HDC(), (action%16)*16, (action/16)*32, 16, 32, SRCCOPY);
					//*/
				}
			}
		}
	}else{
		buf->select_object(GetStockObject(BLACK_BRUSH));
		buf->rectangle(0, 0,
			min(rtRedraw->right-rtRedraw->left, map_cx*32),
			min(rtRedraw->bottom-rtRedraw->top, map_cy*32));
	}
}

void LevelTilemapManager::write_action_chips(){
	int x, y;
	WORD tile, action;
	HDC hdc_action = action_chip.get_HDC();
	action_chip.select_object(GetStockObject(BLACK_BRUSH));
	action_chip.select_object(GetStockObject(BLACK_PEN));
	action_chip.rectangle(0, 0, 2048, 2048);
	for (int i=0; i<actionmap_size; i++){
		x = (i%16)*32;
		y = (i/16)*32;
		tile = i>=actionmap_size ? 0 : i;
		action = LOWORD(actionmap[tile]);
		draw_action(x, y, action);
		/*
			SetTextColor(hdc_action, RGB(0, 0, 255));
			SetBkMode(hdc_action, TRANSPARENT);
			SelectObject(hdc_action, GetStockObject(DEFAULT_GUI_FONT));
			char s[16];
			sprintf_s(s, sizeof s, "%02X", action&0x7F);
			TextOut(hdc_action, x, y, s, strlen(s));
		//*/
		action = HIWORD(actionmap[tile]);
		draw_action(x+16, y, action);
		/*
			sprintf_s(s, sizeof s, "%02X", action&0x7F);
			TextOut(hdc_action, x+16, y+12, s, strlen(s));
		//*/
		StretchBlt(hdc_action, x+512+31, y, -32, 32, hdc_action, x, y, 32, 32, SRCCOPY);
		StretchBlt(hdc_action, x+1024, y+31, 32,-32, hdc_action, x, y, 32, 32, SRCCOPY);
		StretchBlt(hdc_action, x+1536, y+31, 32,-32, hdc_action, x+512, y, 32, 32, SRCCOPY);
	}
	/*		// `bvƂ̑ꑮt@Cɏo
	BYTE *bPixelBits = new BYTE[4*2048*2048];
	BITMAPINFO bmpInfo;
	ZeroMemory(&bmpInfo, sizeof bmpInfo);
	bmpInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmpInfo.bmiHeader.biWidth = 2048;
	bmpInfo.bmiHeader.biHeight = 2048;
	bmpInfo.bmiHeader.biPlanes = 1;
	bmpInfo.bmiHeader.biBitCount = 32;
	bmpInfo.bmiHeader.biCompression = BI_RGB;
	bmpInfo.bmiHeader.biSizeImage = 4*2048*2048;
	bmpInfo.bmiHeader.biXPelsPerMeter = 0;
	bmpInfo.bmiHeader.biYPelsPerMeter = 0;
	bmpInfo.bmiHeader.biClrUsed = 0;
	bmpInfo.bmiHeader.biClrImportant = 0;
	GetDIBits(
		hdc_action, action_chip.get_bitmap(), 0 , 2048,
		bPixelBits , &bmpInfo, DIB_RGB_COLORS
	);
	HANDLE hFile = CreateFile("out.bmp", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL, NULL);
	DWORD dwBytes;
	BITMAPFILEHEADER bmpFileHeader;
	bmpFileHeader.bfType = 0x4D42;
	bmpFileHeader.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + 4*2048*2048;
	bmpFileHeader.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
	bmpFileHeader.bfReserved1 = 0;
	bmpFileHeader.bfReserved2 = 0;
	WriteFile(hFile, &bmpFileHeader, sizeof(BITMAPFILEHEADER), &dwBytes , NULL);
	WriteFile(hFile, &bmpInfo, sizeof(BITMAPINFOHEADER), &dwBytes, NULL);
	WriteFile(hFile, bPixelBits, 4*2048*2048, &dwBytes, NULL);
	CloseHandle(hFile);
	delete[] bPixelBits;
	//*/
	return;
}

void LevelTilemapManager::draw_action_horizontal(int x, int y, int depth, bool hup, bool p0){
	action_chip.select_object(p0 ? hpnType1 : hpnNormal);
	action_chip.select_object(p0 ? hbrType1 : hbrNormal);
	if (hup){
		MoveToEx(action_chip.get_HDC(), x, y+depth, NULL);
		LineTo(action_chip.get_HDC(), x+16, y+depth);
	}else{
		action_chip.rectangle(x, y+depth, x+16, y+32);
	}
}

void LevelTilemapManager::draw_action_saka(int x, int y, int gnum, int height, bool hup, bool p0, bool hflip){
	static int fx[][17] = {
		{0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3,					4},
		{0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7,					8},
		{0, 1, 1, 2, 3, 3, 4, 5, 5, 6, 7, 7, 8, 9, 10, 11,					12},
		{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 ,15,				16},
		{0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30,			32},
		{0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 52, 56, 60,		64},
		{-1, -1, -1, -1, 0, 4, 8, 12, 16, 16, 16, 16, 16, 16, 16, 16,		16},
		{16, 16, 16, 16, 16, 16, 16, 16, 16, 20, 24, 28, 32, 32, 32, 32,	32},
		{0, 2, 4, 6, 8, 10,	12, 14, 16, 16, 16, 16, 16, 16, 16, 16,			16},
		{16, 16, 16, 16, 16, 16, 16, 16, 16, 18, 20, 22, 24, 26, 28, 30,	32},
	};
	if (gnum<0 || gnum>=10) return;
	int g, gn;
	action_chip.select_object(p0 ? hpnType1 : hpnNormal);
	if (hup){
		for (int i=0; i<16; i++){
			g = fx[gnum][i]+height;
			//if (g < 0 || g >=32) continue;
			if (g<0) continue;
			if (g>=32) g=31;
			gn = fx[gnum][i+1]+height;
			if (gn==g) gn++;
			if (gn>32) gn=32;
			MoveToEx(action_chip.get_HDC(), x+(hflip ? 15-i : i), y+31-g, NULL);
			LineTo(action_chip.get_HDC(), x+(hflip ? 15-i : i), y+31-gn);
		}
	}else{
		for (int i=0; i<16; i++){
			g = fx[gnum][i]+height;
			gn = fx[gnum][i+1]+height;
			if (gn<0) continue;
			if (gn==g) gn++;
			if (gn>32) gn=32;
			MoveToEx(action_chip.get_HDC(), x+(hflip ? 15-i : i), y+31, NULL);
			LineTo(action_chip.get_HDC(), x+(hflip ? 15-i : i), y+31-gn);
		}
	}
}

void LevelTilemapManager::draw_action(int x, int y, WORD action){
	int number = action & 0x007F;
	bool hflip = (action & 0x8000) != 0;
	bool hup = (action & 0x4000) != 0;
	bool p0 = (action & 0x0100) != 0;
	switch (number){
		case 0x00: break;
		case 0x01: draw_action_horizontal(x, y, 0, hup, p0); break;
		case 0x02: draw_action_horizontal(x, y, 4, hup, p0); break;
		case 0x03: draw_action_horizontal(x, y, 8, hup, p0); break;
		case 0x04: draw_action_horizontal(x, y, 12, hup, p0); break;
		case 0x05: draw_action_horizontal(x, y, 16, hup, p0); break;
		case 0x06: draw_action_horizontal(x, y, 20, hup, p0); break;
		case 0x07: draw_action_horizontal(x, y, 24, hup, p0); break;
		case 0x08: draw_action_horizontal(x, y, 28, hup, p0); break;
		case 0x09: draw_action_saka(x, y, 0, 0, hup, p0, hflip); break;
		case 0x0A: draw_action_saka(x, y, 0, 4, hup, p0, hflip); break;
		case 0x0B: draw_action_saka(x, y, 0, 8, hup, p0, hflip); break;
		case 0x0C: draw_action_saka(x, y, 0, 12, hup, p0, hflip); break;
		case 0x0D: draw_action_saka(x, y, 0, 16, hup, p0, hflip); break;
		case 0x0E: draw_action_saka(x, y, 0, 20, hup, p0, hflip); break;
		case 0x0F: draw_action_saka(x, y, 0, 24, hup, p0, hflip); break;
		case 0x10: draw_action_saka(x, y, 0, 28, hup, p0, hflip); break;
		case 0x11: draw_action_saka(x, y, 1, -4, hup, p0, hflip); break;
		case 0x12: draw_action_saka(x, y, 1, 0, hup, p0, hflip); break;
		case 0x13: draw_action_saka(x, y, 1, 4, hup, p0, hflip); break;
		case 0x14: draw_action_saka(x, y, 1, 8, hup, p0, hflip); break;
		case 0x15: draw_action_saka(x, y, 1, 12, hup, p0, hflip); break;
		case 0x16: draw_action_saka(x, y, 1, 16, hup, p0, hflip); break;
		case 0x17: draw_action_saka(x, y, 1, 20, hup, p0, hflip); break;
		case 0x18: draw_action_saka(x, y, 1, 24, hup, p0, hflip); break;
		case 0x19: draw_action_saka(x, y, 1, 28, hup, p0, hflip); break;
		case 0x1A: draw_action_saka(x, y, 2, -8, hup, p0, hflip); break;
		case 0x1B: draw_action_saka(x, y, 2, -4, hup, p0, hflip); break;
		case 0x1C: draw_action_saka(x, y, 2, 0, hup, p0, hflip); break;
		case 0x1D: draw_action_saka(x, y, 2, 4, hup, p0, hflip); break;
		case 0x1E: draw_action_saka(x, y, 2, 8, hup, p0, hflip); break;
		case 0x1F: draw_action_saka(x, y, 2, 12, hup, p0, hflip); break;
		case 0x20: draw_action_saka(x, y, 2, 16, hup, p0, hflip); break;
		case 0x21: draw_action_saka(x, y, 2, 20, hup, p0, hflip); break;
		case 0x22: draw_action_saka(x, y, 2, 24, hup, p0, hflip); break;
		case 0x23: draw_action_saka(x, y, 2, 28, hup, p0, hflip); break;
		case 0x24: draw_action_saka(x, y, 3, -12, hup, p0, hflip); break;
		case 0x25: draw_action_saka(x, y, 3, -8, hup, p0, hflip); break;
		case 0x26: draw_action_saka(x, y, 3, -4, hup, p0, hflip); break;
		case 0x27: draw_action_saka(x, y, 3, 0, hup, p0, hflip); break;
		case 0x28: draw_action_saka(x, y, 3, 4, hup, p0, hflip); break;
		case 0x29: draw_action_saka(x, y, 3, 8, hup, p0, hflip); break;
		case 0x2A: draw_action_saka(x, y, 3, 12, hup, p0, hflip); break;
		case 0x2B: draw_action_saka(x, y, 3, 16, hup, p0, hflip); break;
		case 0x2C: draw_action_saka(x, y, 3, 20, hup, p0, hflip); break;
		case 0x2D: draw_action_saka(x, y, 3, 24, hup, p0, hflip); break;
		case 0x2E: draw_action_saka(x, y, 3, 28, hup, p0, hflip); break;
		case 0x2F: draw_action_saka(x, y, 4, -24, hup, p0, hflip); break;
		case 0x30: draw_action_saka(x, y, 4, -16, hup, p0, hflip); break;
		case 0x31: draw_action_saka(x, y, 4, -8, hup, p0, hflip); break;
		case 0x32: draw_action_saka(x, y, 4, 0, hup, p0, hflip); break;
		case 0x33: draw_action_saka(x, y, 4, 8, hup, p0, hflip); break;
		case 0x34: draw_action_saka(x, y, 4, 16, hup, p0, hflip); break;
		case 0x35: draw_action_saka(x, y, 4, 24, hup, p0, hflip); break;
		case 0x36: draw_action_saka(x, y, 5, -48, hup, p0, hflip); break;
		case 0x37: draw_action_saka(x, y, 5, -32, hup, p0, hflip); break;
		case 0x38: draw_action_saka(x, y, 5, -16, hup, p0, hflip); break;
		case 0x39: draw_action_saka(x, y, 5, 0, hup, p0, hflip); break;
		case 0x3A: draw_action_saka(x, y, 5, 16, hup, p0, hflip); break;
		case 0x3B: draw_action_saka(x, y, 6, 0, hup, p0, hflip); break;
		case 0x3C: draw_action_saka(x, y, 7, 0, hup, p0, hflip); break;
		case 0x3D: draw_action_horizontal(x, y, 0, hup, p0); break;
		case 0x3E: draw_action_horizontal(x, y, 16, hup, p0); break;
		case 0x3F:
			action_chip.select_object(p0 ? hpnType1 : hpnNormal);
			action_chip.select_object(p0 ? hbrType1 : hbrNormal);
			if (hup){
				MoveToEx(action_chip.get_HDC(), hflip?x+12:x, y+12, NULL);
				LineTo(action_chip.get_HDC(), hflip?x+16:x+4, y+12);
			}else{
				action_chip.rectangle(hflip?x+12:x, y+12, hflip?x+16:x+4, y+32);
			}
			action_chip.select_object(hpnType3);
			action_chip.select_object(hbrType3);
			action_chip.rectangle(hflip?x:x+4, y+12, hflip?x+12:x+16, y+32);
			break;
		case 0x40:
			action_chip.select_object(hpnType3);
			action_chip.select_object(hbrType3);
			action_chip.rectangle(x, y+10, x+16, y+22);
			break;
		case 0x41:
			action_chip.select_object(hpnType3);
			action_chip.select_object(hbrType3);
			action_chip.rectangle(hflip?x:x+4, y, hflip?x+12:x+16, y+32);
			break;
		case 0x42:
			action_chip.select_object(hpnType3);
			action_chip.select_object(hbrType3);
			action_chip.rectangle(hflip?x+12:x, y+10, hflip?x+16:x+4, y+22);
			action_chip.rectangle(hflip?x:x+4, y, hflip?x+12:x+16, y+32);
			break;
		case 0x43:
			action_chip.select_object(hpnType3);
			action_chip.select_object(hbrType3);
			action_chip.rectangle(hflip?x:x+4, y, hflip?x+12:x+16, y+24);
			break;
		case 0x44:
			action_chip.select_object(hpnType3);
			action_chip.select_object(hbrType3);
			action_chip.rectangle(hflip?x:x+4, y, hflip?x+12:x+16, y+16);
			break;
		case 0x45:
			action_chip.select_object(hpnType3);
			action_chip.select_object(hbrType3);
			action_chip.rectangle(hflip?x:x+4, y, hflip?x+12:x+16, y+8);
			break;
		case 0x46:
			action_chip.select_object(p0 ? hpnType1 : hpnNormal);
			action_chip.select_object(p0 ? hbrType1 : hbrNormal);
			if (hup){
				MoveToEx(action_chip.get_HDC(), hflip?x+12:x, y+12, NULL);
				LineTo(action_chip.get_HDC(), hflip?x+16:x+4, y+12);
			}else{
				action_chip.rectangle(hflip?x+12:x, y+12, hflip?x+16:x+4, y+32);
			}
			action_chip.select_object(hpnType3);
			action_chip.select_object(hbrType3);
			action_chip.rectangle(hflip?x:x+4, y, hflip?x+12:x+16, y+32);
			break;
		case 0x47:
			action_chip.select_object(hpnType2);
			action_chip.select_object(hbrType2);
			if (hup){
				MoveToEx(action_chip.get_HDC(), x, y, NULL);
				LineTo(action_chip.get_HDC(), x+16, y);
			}else{
				action_chip.rectangle(x, y, x+16, y+32);
			}
			break;
		case 0x48:
			action_chip.select_object(hpnType2);
			action_chip.select_object(hbrType2);
			if (hup){
				MoveToEx(action_chip.get_HDC(), x, y+16, NULL);
				LineTo(action_chip.get_HDC(), x+16, y+16);
			}else{
				action_chip.rectangle(x, y+16, x+16, y+32);
			}
			break;
		case 0x49: draw_action_saka(x, y, 8, 0, hup, p0, hflip); break;
		case 0x4A: draw_action_saka(x, y, 9, 0, hup, p0, hflip); break;
		case 0x4B:
			action_chip.select_object(p0 ? hpnType1 : hpnNormal);
			action_chip.select_object(p0 ? hbrType1 : hbrNormal);
			for (int i=0; i<16; i++){
				MoveToEx(action_chip.get_HDC(), x+(hflip ? 15-i : i), y, NULL);
				LineTo(action_chip.get_HDC(), x+(hflip ? 15-i : i), y+16+15-i);
			}
			break;
		case 0x4C:
			action_chip.select_object(p0 ? hpnType1 : hpnNormal);
			action_chip.select_object(p0 ? hbrType1 : hbrNormal);
			for (int i=0; i<16; i++){
				MoveToEx(action_chip.get_HDC(), x+(hflip ? 15-i : i), y, NULL);
				LineTo(action_chip.get_HDC(), x+(hflip ? 15-i : i), y+15-i);
			}
			break;
		case 0x4D:
			action_chip.select_object(p0 ? hpnType1 : hpnNormal);
			action_chip.select_object(p0 ? hbrType1 : hbrNormal);
			action_chip.rectangle(x, y, x+16, y+16);
			break;
		case 0x4E:
			action_chip.select_object(p0 ? hpnType1 : hpnNormal);
			action_chip.select_object(p0 ? hbrType1 : hbrNormal);
			for (int i=0; i<16; i++){
				MoveToEx(action_chip.get_HDC(), x+(hflip ? 15-i : i), y, NULL);
				LineTo(action_chip.get_HDC(), x+(hflip ? 15-i : i), y+(15-i)*2);
			}
			break;
		case 0x4F:
			action_chip.select_object(p0 ? hpnType1 : hpnNormal);
			action_chip.select_object(p0 ? hbrType1 : hbrNormal);
			if (hup){
				MoveToEx(action_chip.get_HDC(), hflip?x+8:x, y, NULL);
				LineTo(action_chip.get_HDC(), hflip?x+16:x+8, y);
			}else{
				action_chip.rectangle(hflip?x+8:x, y, hflip?x+16:x+8, y+32);
			}
			break;
	}
	return;
}


BOOL LevelTilemapManager::command(WPARAM wParam, LPARAM lParam){
	if (LOWORD(wParam) == IDM_VIEW_TILEMAP){
		switch (iShowTilemap){
			case MAP_HIDE:
				iShowTilemap = MAP_TILEMAP;
				break;
			case MAP_TILEMAP:
				iShowTilemap = MAP_ACTION;
				break;
			case MAP_ACTION:
				iShowTilemap = MAP_HIDE;
				break;
		}
		set_menuitem_state();
		level->redraw_default();
		return TRUE;
	}
	if (LOWORD(wParam) == IDM_VIEW_MAP_HIDE){
		iShowTilemap = MAP_HIDE;
		set_menuitem_state();
		level->redraw_default();
		return TRUE;
	}
	if (LOWORD(wParam) == IDM_VIEW_MAP_TILEMAP){
		iShowTilemap = MAP_TILEMAP;
		set_menuitem_state();
		level->redraw_default();
		return TRUE;
	}
	if (LOWORD(wParam) == IDM_VIEW_MAP_ACTION){
		iShowTilemap = MAP_ACTION;
		set_menuitem_state();
		level->redraw_default();
		return TRUE;
	}
	return FALSE;
}






