
///////////////////////////////////////////////////////////////////////////////
//
//	project.h
//

#ifndef _PROJECT_
#define _PROJECT_

///////////////////////////////////////////////////////////////////////////////

#include <deque>
#include <stack>
#include <string>
#include <windows.h>
#include "base.h"
#include "window.h"
#include "level.h"
#include "undo.h"

#define ROM_SIZE_NO_HEADER	0x400000
#define ROM_HEADER_SIZE		0x000200

class ProjectManager;


//	UndoBuffer: AhDEhD̊Ǘ
//---------------------------------------------------------
class UndoBuffer{
	std::stack<UndoMethod*> undo_buf;
	std::stack<UndoMethod*> redo_buf;
	void clear_redo();
	ProjectManager *project;
	HMENU hmEdit;
public:
	UndoBuffer();
	~UndoBuffer();
	BOOL create(ProjectManager *_project);
	void add_method(UndoMethod *undo);
	void clear();
	bool undo();
	bool redo();
	void reset_menuitem() const;
	BOOL command(WPARAM wParam, LPARAM lParam);
};


//	RecentFileManager: ŋߊJt@C̊Ǘ
//---------------------------------------------------------
class RecentFileManager : public UI_Base{
	typedef std::deque<std::string> Filenames;
	static const int FILES_MAX=16;
	ProjectManager *project;
	Filenames files;
public:
	RecentFileManager(ProjectManager *_project);
	BOOL create();
	BOOL command(WPARAM wParam, LPARAM lParam);
	void file_opened(LPCSTR filename);
	void clear();
	void save_data();
};


//	TestPlayManager: eXgvC̊Ǘ
//---------------------------------------------------------
class TestPlayManager : public UI_Base{
	ProjectManager *project;
	void test_play();
	void set_emulator();
public:
	TestPlayManager(ProjectManager *_project);
	BOOL command(WPARAM wParam, LPARAM lParam);
};


//	ProjectManager: vWFNgSʂ̊ǗyуCEBhE̊Ǘ
//---------------------------------------------------------
class ProjectManager : public MyWindow, public RomData_Base{
private:
	static LPCSTR pAppNameMainWindow;
	static LPCSTR pClassName;
public:
	static BOOL Init();

	bool bEnabled;
	int nChangeStep;
	bool bChangedInrev;
	char filename[MAX_PATH];
	char filetitle[270];
	bool bInterleaved;
	bool bHasHeader;
	BYTE *header;
	BYTE *buf;

public:
	
	RecentFileManager recent_files;
	UndoBuffer undo_buffer;
	LevelManager levels;
	TestPlayManager test_play;

	bool read_data(void *dest, size_t nSize, LADRS adrs) const;
	bool read_data(void *dest, size_t nSize, BANK bank, SADRS adrs) const;
	SADRS read_sadrs(LADRS adrs) const;
	SADRS read_sadrs(BANK bank, SADRS adrs) const;
	LADRS read_ladrs_normal(LADRS adrs) const;
	LADRS read_ladrs_normal(BANK bank, SADRS adrs) const;
	BYTE read_byte(LADRS adrs) const;
	BYTE read_byte(BANK bank, SADRS adrs) const;
	WORD read_word(LADRS adrs) const;
	WORD read_word(BANK bank, SADRS adrs) const;
	bool write_data(const void *src, size_t nSize, LADRS adrs);
	bool write_data(const void *src, size_t nSize, BANK bank, SADRS adrs);
	void write_sadrs(SADRS data, LADRS adrs);
	void write_sadrs(SADRS data, BANK bank, SADRS adrs);
	void write_ladrs_normal(LADRS data, LADRS adrs);
	void write_ladrs_normal(LADRS data, BANK bank, SADRS adrs);
	void write_byte(BYTE data, LADRS adrs);
	void write_byte(BYTE data, BANK bank, SADRS adrs);
	void write_word(WORD data, LADRS adrs);
	void write_word(WORD data, BANK bank, SADRS adrs);
	void *get_pointer(LADRS adrs) const;
	void *get_pointer(BANK bank, SADRS adrs) const;

private:

	virtual LRESULT Proc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lparam);
	void save_window_pos();
	void set_state(bool _bEnabled);
	BOOL command(WPARAM wParam, LPARAM lParam);
	void open_cmdline(LPCSTR cmdline);
	void open_dropped_files(HDROP hDrop);
	void open();
	bool load();
	void save();
	void save_as();
	void reload();
	void close();
	void quit();
	bool query_saving();

public:

	ProjectManager();
	~ProjectManager();
	BOOL create(LPCSTR cmdline, int nCmdShow);
	void load_file(LPCSTR _filename);
	void nc_activate(WPARAM wParam);
	void changed_normal();
	void changed_undo();
	void changed_redo();

};




///////////////////////////////////////////////////////////////////////////////

#endif // _PROJECT_


