
///////////////////////////////////////////////////////////////////////////////
//
//	scrbar.cpp
//

#include "scrbar.h"
#include "id.h"


WindowWithHScrollBar::WindowWithHScrollBar(){
	si.cbSize			= sizeof si;
	si.nMax				= 0;
	si.nMin				= 0;
	si.nPage			= 0;
	si.nPos				= 0;
	line_size			= 1;
	bDisableNoScroll	= true;
	bRedraw				= true;
}

WindowWithHScrollBar::~WindowWithHScrollBar(){
}

void WindowWithHScrollBar::scrolled_h(int nNewPos, UINT nNewPage, int nOldPos, UINT nOldPage){
	return;
}

void WindowWithHScrollBar::set_disable_no_Scroll_h(bool _bDisableNoScroll){
	bDisableNoScroll = _bDisableNoScroll;
	return;
}

void WindowWithHScrollBar::set_redraw_h(bool _bRedraw){
	bRedraw = _bRedraw;
	return;
}

void WindowWithHScrollBar::set_range_h(int nMin, int nMax){
	si.fMask	= SIF_RANGE | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nMax		= nMax;
	si.nMin		= nMin;
	SetScrollInfo(hWnd, SB_HORZ, &si, bRedraw);
	return;
}

void WindowWithHScrollBar::set_page_size_h(UINT nPage){
	si.fMask	= SIF_PAGE | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPage	= nPage;
	SetScrollInfo(hWnd, SB_HORZ, &si, bRedraw);
	return;
}

void WindowWithHScrollBar::set_line_size_h(UINT nLineSize){
	line_size = nLineSize;
	return;
}

void WindowWithHScrollBar::set_position_h(int nPos){
	si.fMask	= SIF_POS | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPos		= nPos;
	SetScrollInfo(hWnd, SB_HORZ, &si, bRedraw);
	return;
}

void WindowWithHScrollBar::resize_h(UINT nPage){
	int limit;
	int nOldPos = si.nPos;
	UINT nOldPage = si.nPage;
	si.fMask	= SIF_PAGE | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPage	= nPage;
	limit = si.nMax - si.nPage + 1;
	if (si.nPos > limit){
		si.fMask |= SIF_POS;
		si.nPos = (limit >= si.nMin ? limit : si.nMin);
	}
	SetScrollInfo(hWnd, SB_HORZ, &si, bRedraw);
	scrolled_h(si.nPos, si.nPage, nOldPos, nOldPage);
	return;
}

void WindowWithHScrollBar::handle_message_h(WPARAM wParam, LPARAM lParam){
	int limit = si.nMax - si.nPage + 1;
	int nOldPos = si.nPos;
	switch (LOWORD(wParam)){
		case SB_LEFT:
			si.nPos = si.nMin;
			break;
		case SB_RIGHT:
			si.nPos = limit;
			break;
		case SB_LINELEFT:
			si.nPos -= line_size;
			break;
		case SB_LINERIGHT:
			si.nPos += line_size;
			break;
		case SB_PAGELEFT:
			si.nPos -= si.nPage;
			break;
		case SB_PAGERIGHT:
			si.nPos += si.nPage;
			break;
		case SB_THUMBTRACK:
			si.nPos = HIWORD(wParam);
			break;
		default:
			return;
	}
	si.fMask = SIF_POS | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPos = max(si.nMin, min(limit, si.nPos));
	SetScrollInfo(hWnd, SB_HORZ, &si, bRedraw);
	scrolled_h(si.nPos, si.nPage, nOldPos, si.nPage);
	return;
}

void WindowWithHScrollBar::scroll_h(int dx){
	int nOldPos = si.nPos;
	int limit = si.nMax - si.nPage + 1;
	si.fMask = SIF_POS | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPos = max(si.nMin, min(limit, si.nPos + dx));
	SetScrollInfo(hWnd, SB_HORZ, &si, bRedraw);
	scrolled_h(si.nPos, si.nPage, nOldPos, si.nPage);
	return;
}

int WindowWithHScrollBar::get_pos_h() const{
	return si.nPos;
}

UINT WindowWithHScrollBar::get_page_h() const{
	return si.nPage;
}


//-----------------------------------------------------------------------------


WindowWithVScrollBar::WindowWithVScrollBar(){
	si.cbSize			= sizeof si;
	si.nMax				= 0;
	si.nMin				= 0;
	si.nPage			= 0;
	si.nPos				= 0;
	line_size			= 1;
	bDisableNoScroll	= true;
	bRedraw				= true;
}

WindowWithVScrollBar::~WindowWithVScrollBar(){
}

void WindowWithVScrollBar::scrolled_v(int nNewPos, UINT nNewPage, int nOldPos, UINT nOldPage){
	return;
}

void WindowWithVScrollBar::set_disable_no_Scroll_v(bool _bDisableNoScroll){
	bDisableNoScroll = _bDisableNoScroll;
	return;
}

void WindowWithVScrollBar::set_redraw_v(bool _bRedraw){
	bRedraw = _bRedraw;
	return;
}

void WindowWithVScrollBar::set_range_v(int nMin, int nMax){
	si.fMask	= SIF_RANGE | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nMax		= nMax;
	si.nMin		= nMin;
	SetScrollInfo(hWnd, SB_VERT, &si, bRedraw);
	return;
}

void WindowWithVScrollBar::set_page_size_v(UINT nPage){
	si.fMask	= SIF_PAGE | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPage	= nPage;
	SetScrollInfo(hWnd, SB_VERT, &si, bRedraw);
	return;
}

void WindowWithVScrollBar::set_line_size_v(UINT nLineSize){
	line_size = nLineSize;
	return;
}

void WindowWithVScrollBar::set_position_v(int nPos){
	si.fMask	= SIF_POS | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPos		= nPos;
	SetScrollInfo(hWnd, SB_VERT, &si, bRedraw);
	return;
}

void WindowWithVScrollBar::resize_v(UINT nPage){
	int limit;
	int nOldPos = si.nPos;
	UINT nOldPage = si.nPage;
	si.fMask	= SIF_PAGE | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPage	= nPage;
	limit = si.nMax - si.nPage + 1;
	if (si.nPos > limit){
		si.fMask |= SIF_POS;
		si.nPos = (limit >= si.nMin ? limit : si.nMin);
	}
	SetScrollInfo(hWnd, SB_VERT, &si, bRedraw);
	scrolled_v(si.nPos, si.nPage, nOldPos, nOldPage);
	return;
}

void WindowWithVScrollBar::handle_message_v(WPARAM wParam, LPARAM lParam){
	int limit = si.nMax - si.nPage + 1;
	int nOldPos = si.nPos;
	switch (LOWORD(wParam)){
		case SB_TOP:
			si.nPos = si.nMin;
			break;
		case SB_BOTTOM:
			si.nPos = limit;
			break;
		case SB_LINEUP:
			si.nPos -= line_size;
			break;
		case SB_LINEDOWN:
			si.nPos += line_size;
			break;
		case SB_PAGEUP:
			si.nPos -= si.nPage;
			break;
		case SB_PAGEDOWN:
			si.nPos += si.nPage;
			break;
		case SB_THUMBTRACK:
			si.nPos = HIWORD(wParam);
			break;
		default:
			return;
	}
	si.fMask = SIF_POS | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPos = max(si.nMin, min(limit, si.nPos));
	SetScrollInfo(hWnd, SB_VERT, &si, bRedraw);
	scrolled_v(si.nPos, si.nPage, nOldPos, si.nPage);
	return;
}

void WindowWithVScrollBar::scroll_v(int dy){
	int nOldPos = si.nPos;
	int limit = si.nMax - si.nPage + 1;
	si.fMask = SIF_POS | (bDisableNoScroll ? SIF_DISABLENOSCROLL : 0);
	si.nPos = max(si.nMin, min(limit, si.nPos + dy));
	SetScrollInfo(hWnd, SB_VERT, &si, bRedraw);
	scrolled_v(si.nPos, si.nPage, nOldPos, si.nPage);
	return;
}

int WindowWithVScrollBar::get_pos_v() const{
	return si.nPos;
}

UINT WindowWithVScrollBar::get_page_v() const{
	return si.nPage;
}



//-----------------------------------------------------------------------------

MButtonPan::MButtonPan(WindowWithHScrollBar *_pWnd, DWORD _dwPeriod, DWORD _dwID)
: pWnd(_pWnd), dwPeriod(_dwPeriod), dwID(_dwID), hilDragOrg(NULL)
{
	direction = PAN_HORZ;
	window.pWnd_h = _pWnd;
	bEnabled = false;
}
MButtonPan::MButtonPan(WindowWithVScrollBar *_pWnd, DWORD _dwPeriod, DWORD _dwID)
: pWnd(_pWnd), dwPeriod(_dwPeriod), dwID(_dwID), hilDragOrg(NULL)
{
	direction = PAN_VERT;
	window.pWnd_v = _pWnd;
	bEnabled = false;
}
MButtonPan::MButtonPan(WindowWithBothScrollBars *_pWnd, DWORD _dwPeriod, DWORD _dwID)
: pWnd(_pWnd), dwPeriod(_dwPeriod), dwID(_dwID), hilDragOrg(NULL)
{
	direction = PAN_BOTH;
	window.pWnd_both = _pWnd;
	bEnabled = false;
}

MButtonPan::~MButtonPan(){
	if (bEnabled) stop();
}


void MButtonPan::start(POINT cursor){
	cursor_p = cursor;
	bEnabled = true;
	SetCapture(pWnd->get_hWnd());
	SetCursor(LoadCursor(NULL, IDC_ARROW));
	hilDragOrg = ImageList_Create(32, 32, ILC_COLOR32 | ILC_MASK, 1, 0);
	HICON hIcon;
	switch (direction){
		case PAN_HORZ: hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_PAN_HORZ)); break;
		case PAN_VERT: hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_PAN_VERT)); break;
		case PAN_BOTH: hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_PAN_BOTH)); break;
	}
	ImageList_AddIcon(hilDragOrg, hIcon);
	ImageList_BeginDrag(hilDragOrg, 0, 15, 15);
	ClientToScreen(pWnd->get_hWnd(), &cursor);
	ImageList_DragEnter(GetDesktopWindow(), cursor.x, cursor.y);
	SetTimer(pWnd->get_hWnd(), dwID, dwPeriod, NULL);
	return;
}

void MButtonPan::stop(){
	if (bEnabled){
		KillTimer(pWnd->get_hWnd(), dwID);
		ImageList_DragLeave(GetDesktopWindow());
		ImageList_EndDrag();
		ImageList_Destroy(hilDragOrg);
		ReleaseCapture();
		bEnabled = false;
	}
	return;
}

bool MButtonPan::handle_mousemove(POINT cursor){
	if (!bEnabled) return false;
	return true;
}

bool MButtonPan::handle_timer(DWORD _dwID){
	if (_dwID != dwID || !bEnabled) return false;
	POINT pt;
	GetCursorPos(&pt);
	ScreenToClient(pWnd->get_hWnd(), &pt);
	int dx = (pt.x - cursor_p.x);
	int dy = (pt.y - cursor_p.y);
	dx = dx * (abs(dx)+32) / 1024;
	dy = dy * (abs(dy)+32) / 1024;
	switch (direction){
		case PAN_HORZ:
			window.pWnd_h->scroll_h(dx);
			break;
		case PAN_VERT:
			window.pWnd_v->scroll_v(dy);
			break;
		case PAN_BOTH:
			window.pWnd_both->scroll_h(dx);
			window.pWnd_both->scroll_v(dy);
			break;
	}
	POINT cursor_scr = cursor_p;
	ClientToScreen(pWnd->get_hWnd(), &cursor_scr);
	ImageList_DragMove(cursor_scr.x, cursor_scr.y);
	return true;
}








