
///////////////////////////////////////////////////////////////////////////////
//
//	undo_buf.cpp
//

#include <stdio.h>
#include "id.h"
#include "project.h"

UndoBuffer::UndoBuffer() : project(NULL), hmEdit(NULL){
}

UndoBuffer::~UndoBuffer(){
	clear();
}

BOOL UndoBuffer::create(ProjectManager *_project){
	project = _project;
	hmEdit = GetSubMenu(GetMenu(project->get_hWnd()), 1);
	return TRUE;
}

void UndoBuffer::reset_menuitem() const{
	MENUITEMINFO mi;
	char text[256];
	ZeroMemory(&mi, sizeof mi);
	mi.cbSize			= sizeof mi;
	mi.fMask			= MIIM_STATE | MIIM_TYPE;
	mi.fType			= MFT_STRING;
	if (undo_buf.empty()){
		mi.fState		= MFS_GRAYED;
		mi.dwTypeData	= "Undo\tCtrl+Z";
	}else{
		mi.fState		= MFS_ENABLED;
		sprintf_s(text, sizeof text, "Undo\tCtrl+Z", undo_buf.top()->text_undo());
		mi.dwTypeData	= text;
	}
	SetMenuItemInfo(hmEdit, IDM_EDIT_UNDO, FALSE, &mi);
	if (redo_buf.empty()){
		mi.fState		= MFS_GRAYED;
		mi.dwTypeData	= "Redo\tCtrl+Y";
	}else{
		mi.fState		= MFS_ENABLED;
		sprintf_s(text, sizeof text, "Redo\tCtrl+Y", redo_buf.top()->text_redo());
		mi.dwTypeData	= text;
	}
	SetMenuItemInfo(hmEdit, IDM_EDIT_REDO, FALSE, &mi);
	return;
}

void UndoBuffer::clear(){
	while (!redo_buf.empty()){
		delete redo_buf.top();
		redo_buf.pop();
	}
	while (!undo_buf.empty()){
		delete undo_buf.top();
		undo_buf.pop();
	}
	reset_menuitem();
}

void UndoBuffer::clear_redo(){
	while (!redo_buf.empty()){
		delete redo_buf.top();
		redo_buf.pop();
	}
}

void UndoBuffer::add_method(UndoMethod *undo){
	clear_redo();
	undo_buf.push(undo);
	reset_menuitem();
	project->changed_normal();
	return;
}

bool UndoBuffer::undo(){
	if (undo_buf.empty()) return false;
	undo_buf.top()->undo();
	redo_buf.push(undo_buf.top());
	undo_buf.pop();
	reset_menuitem();
	project->changed_undo();
	return true;
}

bool UndoBuffer::redo(){
	if (redo_buf.empty()) return false;
	redo_buf.top()->redo();
	undo_buf.push(redo_buf.top());
	redo_buf.pop();
	reset_menuitem();
	project->changed_redo();
	return true;
}


BOOL UndoBuffer::command(WPARAM wParam, LPARAM lParam){
	switch (LOWORD(wParam)){
		case IDM_EDIT_UNDO:
			undo();
			return TRUE;
		case IDM_EDIT_REDO:
			redo();
			return TRUE;
	}
	return FALSE;
}







