
//////////////////////////////////////////////////////////////////////////
//
//	window.cpp
//

#include <stdio.h>
#include "window.h"


//	Base
//------------------------------------------------------------------------
char Base::ExeDirectory[] = "";
void Base::Init(){
	GetModuleFileName(NULL, ExeDirectory, sizeof ExeDirectory);
	char *last_sep = strrchr(ExeDirectory, '\\');
	*(last_sep + 1) = '\0';
	return;
}


//	RomData_Base
//------------------------------------------------------------------------
char RomData_Base::RomInfoFileName[] = "";
char RomData_Base::SpriteInfoFileName[] = "";
void RomData_Base::Init(){
	sprintf_s(RomInfoFileName, sizeof RomInfoFileName, "%srominfo.ini", ExeDirectory);
	sprintf_s(SpriteInfoFileName, sizeof SpriteInfoFileName, "%ssprites.ini", ExeDirectory);
	return;
}


//	UI_Base
//------------------------------------------------------------------------

HINSTANCE UI_Base::hInst = NULL;
char UI_Base::ConfigFileName[] = "";
void UI_Base::Init(HINSTANCE _hInst){
	hInst = _hInst;
	sprintf_s(ConfigFileName, sizeof ConfigFileName, "%sconfig.ini", ExeDirectory);
	return;
}


//	Window
//------------------------------------------------------------------------


Window::Window(){
	hWnd = NULL;
}

Window::~Window(){
}

HWND Window::get_hWnd(){
	return hWnd;
}

BOOL Window::create_window_ex(LPCSTR lpszClassName, LPCSTR lpszWindowText,
							  DWORD dwStyle, DWORD dwExStyle,
							  LONG x, LONG y, LONG w, LONG h, LPVOID lpParam, Window *pwParent)
{
	hWnd = CreateWindowEx(dwExStyle, lpszClassName, lpszWindowText, dwStyle,
		x, y, w, h, pwParent ? pwParent->get_hWnd() : NULL, NULL, hInst, lpParam);
	return hWnd != NULL;
}

BOOL Window::destroy(){
	return DestroyWindow(hWnd);
}

BOOL Window::move(LONG x, LONG y, LONG w, LONG h, BOOL bRepaint){
	return MoveWindow(hWnd, x, y, w, h, bRepaint);
}

BOOL Window::set_text(LPCSTR lpszText){
	return SetWindowText(hWnd, lpszText);
}

//	MyWindow
//------------------------------------------------------------------------

static LPCSTR lpszPropName = "__BFC_OWN_INSTANCE__";


BOOL MyWindow::register_class_ex(UINT style, HICON hIcon, HICON hIconSm, HCURSOR hCursor, HBRUSH hbrBackground,
LPCSTR lpszClassName, LPCSTR lpszMenuName){
	WNDCLASSEX wc;
	wc.cbSize			= sizeof(wc);
	wc.style			= style;
	wc.lpfnWndProc		= WndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInst;
	wc.hIcon			= hIcon;
	wc.hIconSm			= hIconSm;
	wc.hCursor			= hCursor;
	wc.hbrBackground	= hbrBackground;
	wc.lpszClassName	= lpszClassName;
	wc.lpszMenuName		= lpszMenuName;
	return (BOOL)RegisterClassEx(&wc);
}

BOOL MyWindow::create_window_ex(LPCSTR lpszClassName, LPCSTR lpszWindowText,
							  DWORD dwStyle, DWORD dwExStyle,
							  LONG x, LONG y, LONG w, LONG h, Window *pwParent)
{
	hWnd = CreateWindowEx(dwExStyle, lpszClassName, lpszWindowText, dwStyle,
		x, y, w, h, pwParent ? pwParent->get_hWnd() : NULL, NULL, hInst, (LPVOID)this);
	return hWnd != NULL;
}

BOOL MyWindow::Attach(HWND hWnd){
	if (!(hWnd && bValid)) return FALSE;
	SetProp(hWnd, lpszPropName, (HANDLE)this);
	return TRUE;
}

BOOL MyWindow::Detach(){
	if (!(hWnd && bValid)) return FALSE;
	RemoveProp(hWnd, lpszPropName);
	return TRUE;
}

MyWindow::MyWindow(){
	bValid = FALSE;
}

MyWindow::~MyWindow(){
	if (bValid) Detach();
}

LRESULT CALLBACK MyWindow::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam){
	MyWindow *pWnd = (MyWindow *)GetProp(hWnd, lpszPropName);
	if (msg == WM_DESTROY){
		if (pWnd){
			pWnd->Detach();
			pWnd->bValid = FALSE;
			return pWnd->Proc(hWnd, msg, wParam, lParam);
		}
	}
	if (pWnd){
		return pWnd->Proc(hWnd, msg, wParam, lParam);
	}
	if (msg == WM_CREATE){
		pWnd = (MyWindow *)((LPCREATESTRUCT)lParam)->lpCreateParams;
		if (pWnd){
			pWnd->bValid = TRUE;
			pWnd->hWnd = hWnd;
			pWnd->Attach(hWnd);
			return pWnd->Proc(hWnd, msg, wParam, lParam);
		}
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

LRESULT MyWindow::Proc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam){
	return DefWindowProc(hWnd, msg, wParam, lParam);
}


///////////////////////////////////////////


struct DialogParam{
	MyDialog *pDialog;
	LPARAM lParam;
};



//	Dialog
//------------------------------------------------------------------------

Dialog::Dialog(){
	hWnd = NULL;
}

Dialog::~Dialog(){
}


int Dialog::dialog_box(Window *pwFrame){
	return DialogBox(hInst, get_template(), pwFrame ? pwFrame->get_hWnd() : NULL, NULL);
}

int Dialog::dialog_box(Window *pwFrame, LPARAM lParam){
	return DialogBoxParam(hInst, get_template(), pwFrame ? pwFrame->get_hWnd() : NULL, NULL, lParam);
}


//	MyDialog
//------------------------------------------------------------------------

BOOL MyDialog::Attach(HWND hWnd){
	if (!(hWnd && bValid)) return FALSE;
	SetProp(hWnd, lpszPropName, (HANDLE)this);
	return TRUE;
}

BOOL MyDialog::Detach(){
	if (!(hWnd && bValid)) return FALSE;
	RemoveProp(hWnd, lpszPropName);
	return TRUE;
}

MyDialog::MyDialog(){
	bValid = FALSE;
}

MyDialog::~MyDialog(){
	if (bValid) Detach();
}

//	{vV[W
BOOL CALLBACK MyDialog::DlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam){
	MyDialog *pWnd = (MyDialog *)GetProp(hWnd, lpszPropName);
	if (msg == WM_DESTROY){
		if (pWnd){
			pWnd->Detach();
			pWnd->bValid = FALSE;
			return pWnd->Proc(hWnd, msg, wParam, lParam);
		}
	}
	if (pWnd){
		return pWnd->Proc(hWnd, msg, wParam, lParam);
	}
	if (msg == WM_INITDIALOG){
		pWnd = ((DialogParam *)lParam)->pDialog;
		if (pWnd){
			pWnd->bValid = TRUE;
			pWnd->hWnd = hWnd;
			pWnd->Attach(hWnd);
			return pWnd->Proc(hWnd, msg, wParam, ((DialogParam *)lParam)->lParam);
		}
	}
	return FALSE;
}

//	ftHg̃TuvV[W
BOOL MyDialog::Proc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam){
	return FALSE;
}

//	[__CAO{bNX
int MyDialog::dialog_box(Window *pwFrame){
	DialogParam dp;
	dp.pDialog = this;
	dp.lParam = 0L;
	return DialogBoxParamA(hInst, get_template(), pwFrame ? pwFrame->get_hWnd() : NULL, DlgProc, (LPARAM)&dp);
}

int MyDialog::dialog_box(Window *pwFrame, LPARAM lParam){
	DialogParam dp;
	dp.pDialog = this;
	dp.lParam = lParam;
	return DialogBoxParamA(hInst, get_template(), pwFrame ? pwFrame->get_hWnd() : NULL, DlgProc, (LPARAM)&dp);
}

int MyDialog::dialog_box(HWND hwParent){
	DialogParam dp;
	dp.pDialog = this;
	dp.lParam = 0L;
	return DialogBoxParamA(hInst, get_template(), hwParent, DlgProc, (LPARAM)&dp);
}

int MyDialog::dialog_box(HWND hwParent, LPARAM lParam){
	DialogParam dp;
	dp.pDialog = this;
	dp.lParam = lParam;
	return DialogBoxParamA(hInst, get_template(), hwParent, DlgProc, (LPARAM)&dp);
}


// [hX_CAO{bNX
BOOL MyDialog::create_dialog(Window *pwFrame){
	DialogParam dp;
	dp.pDialog = this;
	dp.lParam = 0L;
	hWnd = CreateDialogParam(hInst, get_template(), pwFrame ? pwFrame->get_hWnd() : NULL, DlgProc, (LPARAM)&dp);
	return hWnd != NULL;
}

BOOL MyDialog::create_dialog(Window *pwFrame, LPARAM lParam){
	DialogParam dp;
	dp.pDialog = this;
	dp.lParam = lParam;
	hWnd = CreateDialogParam(hInst, get_template(), pwFrame ? pwFrame->get_hWnd() : NULL, DlgProc, (LPARAM)&dp);
	return hWnd != NULL;
}

BOOL MyDialog::create_dialog(HWND hwParent){
	DialogParam dp;
	dp.pDialog = this;
	dp.lParam = 0L;
	hWnd = CreateDialogParam(hInst, get_template(), hwParent, DlgProc, (LPARAM)&dp);
	return hWnd != NULL;
}

BOOL MyDialog::create_dialog(HWND hwParent, LPARAM lParam){
	DialogParam dp;
	dp.pDialog = this;
	dp.lParam = lParam;
	hWnd = CreateDialogParam(hInst, get_template(), hwParent, DlgProc, (LPARAM)&dp);
	return hWnd != NULL;
}



